@@ -5,14 +5,15 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
- *                Jan Treibig (jt), jan.treibig@gmail.com
+ *                Jan Treibig (jt), jan.treibig@gmail.com,
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -50,93 +51,49 @@
 #include <lock.h>
 #include <accessClient_types.h>
 
+
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 #define SA struct sockaddr
 #define str(x) #x
 
 #define CHECK_ERROR(func, msg)  \
-    if ((func) < 0) { \
-        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
-    }
+    if ((func) < 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
 
 #define CHECK_FILE_ERROR(func, msg)  \
-    if ((func) == 0) { \
-        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
-    }
+    if ((func) == 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
 
 
 #define EXIT_IF_ERROR(func, msg)  \
-    if ((func) < 0) { \
-        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
-        stop_daemon(); \
-        exit(EXIT_FAILURE); \
-    }
+    if ((func) < 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); stop_daemon(); exit(EXIT_FAILURE); }
 
 
-#define CPUID \
-    __asm__ volatile ("cpuid" \
-            : "=a" (eax), "=b" (ebx) \
+#define CPUID                    \
+    __asm__ volatile ("cpuid"    \
+            : "=a" (eax),            \
+            "=b" (ebx)             \
             : "0" (eax))
 
 
-/* Intel P6 */
-#define PENTIUM_M_BANIAS     0x09U
-#define PENTIUM_M_DOTHAN     0x0DU
-#define CORE_DUO             0x0EU
-#define CORE2_65             0x0FU
-#define CORE2_45             0x17U
-#define ATOM                 0x1CU
-#define ATOM_45              0x26U
-#define ATOM_32              0x36U
-#define ATOM_22              0x27U
-#define ATOM_SILVERMONT      0x4DU
-#define NEHALEM              0x1AU
-#define NEHALEM_BLOOMFIELD   0x1AU
-#define NEHALEM_LYNNFIELD    0x1EU
-#define NEHALEM_LYNNFIELD_M  0x1FU
-#define NEHALEM_WESTMERE     0x2CU
-#define NEHALEM_WESTMERE_M   0x25U
-#define SANDYBRIDGE          0x2AU
-#define SANDYBRIDGE_EP       0x2DU
-#define HASWELL              0x3CU
-#define HASWELL_EX           0x3FU
-#define HASWELL_M1           0x45U
-#define HASWELL_M2           0x46U
-#define IVYBRIDGE            0x3AU
-#define IVYBRIDGE_EP         0x3EU
-#define NEHALEM_EX           0x2EU
-#define WESTMERE_EX          0x2FU
-#define XEON_MP              0x1DU
-
-/* Intel MIC */
-#define XEON_PHI           0x01U
-
-/* AMD K10 */
-#define BARCELONA      0x02U
-#define SHANGHAI       0x04U
-#define ISTANBUL       0x08U
-#define MAGNYCOURS     0x09U
-
-/* AMD K8 */
-#define OPTERON_SC_1MB  0x05U
-#define OPTERON_DC_E    0x21U
-#define OPTERON_DC_F    0x41U
-#define ATHLON64_X2     0x43U
-#define ATHLON64_X2_F   0x4BU
-#define ATHLON64_F1     0x4FU
-#define ATHLON64_F2     0x5FU
-#define ATHLON64_X2_G   0x6BU
-#define ATHLON64_G1     0x6FU
-#define ATHLON64_G2     0x7FU
-
-
 #define  P6_FAMILY        0x6U
-#define  MIC_FAMILY       0xBU
-#define  NETBURST_FAMILY  0xFFU
+#define  K8_FAMILY        0xFU
+#define  K10_FAMILY       0x10U
 #define  K15_FAMILY       0x15U
 #define  K16_FAMILY       0x16U
-#define  K10_FAMILY       0x10U
-#define  K8_FAMILY        0xFU
+
+#define SANDYBRIDGE          0x2AU
+#define SANDYBRIDGE_EP       0x2DU
+#define IVYBRIDGE            0x3AU
+#define IVYBRIDGE_EP         0x3EU
+#define HASWELL              0x3CU
+#define HASWELL_EP           0x3FU
+#define ATOM_SILVERMONT_E    0x37U
+#define ATOM_SILVERMONT_C    0x4DU
+#define ATOM_SILVERMONT_Z1   0x4AU
+#define ATOM_SILVERMONT_Z2   0x5AU
+#define ATOM_SILVERMONT_F    0x5DU
+#define BROADWELL            0x3DU
+#define BROADWELL_E          0x4FU
+#define BROADWELL_D          0x56U
 
 #define PCI_ROOT_PATH    "/proc/bus/pci/"
 #define MAX_PATH_LENGTH   60
@@ -156,25 +113,9 @@ static char* filepath;
 static const char* ident = "accessD";
 static FuncPrototype allowed = NULL;
 static int FD_MSR[MAX_NUM_THREADS];
-static int FD_PCI[MAX_NUM_NODES][MAX_NUM_DEVICES];
+static int FD_PCI[MAX_NUM_NODES][MAX_NUM_PCI_DEVICES];
 static int isPCIUncore = 0;
-
-static char* pci_DevicePath[MAX_NUM_DEVICES] = {
- "13.5",   /* PCI_R3QPI_DEVICE_LINK_0 */
- "13.6",   /* PCI_R3QPI_DEVICE_LINK_1 */
- "13.1",   /* PCI_R2PCIE_DEVICE */
- "10.0",   /* PCI_IMC_DEVICE_CH_0 */
- "10.1",   /* PCI_IMC_DEVICE_CH_1 */
- "10.4",   /* PCI_IMC_DEVICE_CH_2 */
- "10.5",   /* PCI_IMC_DEVICE_CH_3 */
- "0e.1",   /* PCI_HA_DEVICE */
- "08.2",   /* PCI_QPI_DEVICE_PORT_0 */
- "09.2",   /* PCI_QPI_DEVICE_PORT_1 */
- "08.6",   /* PCI_QPI_MASK_DEVICE_PORT_0 */
- "09.6",   /* PCI_QPI_MASK_DEVICE_PORT_1 */
- "08.0",   /* PCI_QPI_MISC_DEVICE_PORT_0 */
- "09.0" }; /* PCI_QPI_MISC_DEVICE_PORT_1 */
-
+static PciDevice* pci_devices = NULL;
 static char pci_filepath[MAX_PATH_LENGTH];
 
 /* Socket to bus mapping -- will be determined at runtime;
@@ -185,9 +126,79 @@ static char pci_filepath[MAX_PATH_LENGTH];
  *   2                  0xbf
  *   3                  0xff
  */
-static char* socket_bus[MAX_NUM_NODES];
+static char* socket_bus[MAX_NUM_NODES] = { [0 ... (MAX_NUM_NODES-1)] = NULL};
 
 
+static PciDevice sandybridgeEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
+ [MSR_DEV] = {NONE, NULL, NULL, NULL, 0x0, 0},
+ [PCI_R3QPI_DEVICE_LINK_0] = {R3QPI, "13.5", "PCI_R3QPI_DEVICE_LINK_0", "RBOX0", 0x3c44, 0},
+ [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "13.6", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x3c45, 0},
+ [PCI_R2PCIE_DEVICE] = {R2PCIE, "13.1", "PCI_R2PCIE_DEVICE", "PBOX0", 0x3c43, 0},
+ [PCI_IMC_DEVICE_0_CH_0] = {IMC, "10.0", "PCI_IMC_DEVICE_CH_0", "MBOX0", 0x3cb0, 0},
+ [PCI_IMC_DEVICE_0_CH_1] = {IMC, "10.1", "PCI_IMC_DEVICE_CH_1", "MBOX1", 0x3cb1, 0},
+ [PCI_IMC_DEVICE_0_CH_2] = {IMC, "10.4", "PCI_IMC_DEVICE_CH_2", "MBOX2", 0x3cb4, 0},
+ [PCI_IMC_DEVICE_0_CH_3] = {IMC, "10.5", "PCI_IMC_DEVICE_CH_3", "MBOX3", 0x3cb5, 0},
+ [PCI_HA_DEVICE_0] = {HA, "0e.1", "PCI_HA_DEVICE", "BBOX", 0x3c46, 0},
+ [PCI_QPI_DEVICE_PORT_0] = {QPI, "08.2", "PCI_QPI_DEVICE_PORT_0", "SBOX0", 0x3c41, 0},
+ [PCI_QPI_DEVICE_PORT_1] = {QPI, "09.2", "PCI_QPI_DEVICE_PORT_1", "SBOX1", 0x3c42, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_0] = {QPI, "08.6", "PCI_QPI_MASK_DEVICE_PORT_0", NULL, 0x3c86, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_1] = {QPI, "09.6", "PCI_QPI_MASK_DEVICE_PORT_1", NULL, 0x3c96, 0},
+ [PCI_QPI_MISC_DEVICE_PORT_0] = {QPI, "08.0", "PCI_QPI_MISC_DEVICE_PORT_0", "SBOX0FIX",0x3c80, 0},
+ [PCI_QPI_MISC_DEVICE_PORT_1] = {QPI, "09.0", "PCI_QPI_MISC_DEVICE_PORT_1", "SBOX1FIX", 0x3c91, 0},
+};
+
+
+static PciDevice ivybridgeEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
+ [MSR_DEV] = {NONE, "", "", "", 0x0, 0},
+ [PCI_R3QPI_DEVICE_LINK_0] = {R3QPI, "13.5", "PCI_R3QPI_DEVICE_LINK_0", "RBOX0", 0x0e36, 0},
+ [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "13.6", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x0e37, 0},
+ [PCI_R3QPI_DEVICE_LINK_2] = {R3QPI, "12.5", "PCI_R3QPI_DEVICE_LINK_2", "RBOX2", 0x0e3e, 0},
+ [PCI_R2PCIE_DEVICE] = {R2PCIE, "13.1", "PCI_R2PCIE_DEVICE", "PBOX0", 0x0e34, 0},
+ [PCI_IMC_DEVICE_0_CH_0] = {IMC, "10.4", "PCI_IMC_DEVICE_0_CH_0", "MBOX0", 0x0eb4, 0},
+ [PCI_IMC_DEVICE_0_CH_1] = {IMC, "10.5", "PCI_IMC_DEVICE_0_CH_1", "MBOX1", 0x0eb5, 0},
+ [PCI_IMC_DEVICE_0_CH_2] = {IMC, "10.0", "PCI_IMC_DEVICE_0_CH_2", "MBOX2", 0x0eb0, 0},
+ [PCI_IMC_DEVICE_0_CH_3] = {IMC, "10.1", "PCI_IMC_DEVICE_0_CH_3", "MBOX3", 0x0eb1, 0},
+ [PCI_HA_DEVICE_0] = {HA, "0e.1", "PCI_HA_DEVICE_0", "BBOX0", 0x0e30, 0},
+ [PCI_HA_DEVICE_1] = {HA, "1c.1", "PCI_HA_DEVICE_1", "BBOX1", 0x0e38, 0},
+ [PCI_IMC_DEVICE_1_CH_0] = {IMC, "1e.4", "PCI_IMC_DEVICE_1_CH_0", "MBOX0", 0x0ef4, 0},
+ [PCI_IMC_DEVICE_1_CH_1] = {IMC, "1e.5", "PCI_IMC_DEVICE_1_CH_1", "MBOX1", 0x0ef5, 0},
+ [PCI_IMC_DEVICE_1_CH_2] = {IMC, "1e.0", "PCI_IMC_DEVICE_1_CH_2", "MBOX2", 0x0ef0, 0},
+ [PCI_IMC_DEVICE_1_CH_3] = {IMC, "1e.1", "PCI_IMC_DEVICE_1_CH_3", "MBOX3", 0x0ef1, 0},
+ [PCI_IRP_DEVICE] = {IRP, "05.6", "PCI_IRP_DEVICE", NULL, 0x0e39, 0},
+ [PCI_QPI_DEVICE_PORT_0] = {QPI, "08.2", "PCI_QPI_DEVICE_PORT_0", "SBOX0", 0x0e32, 0},
+ [PCI_QPI_DEVICE_PORT_1] = {QPI, "09.2", "PCI_QPI_DEVICE_PORT_1", "SBOX1", 0x0e33, 0},
+ [PCI_QPI_DEVICE_PORT_2] = {QPI, "0a.2", "PCI_QPI_DEVICE_PORT_2", "SBOX2", 0x0e3a, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_0] = {QPI, "08.6", "PCI_QPI_MASK_DEVICE_PORT_0", NULL, 0x0e86, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_1] = {QPI, "09.6", "PCI_QPI_MASK_DEVICE_PORT_1", NULL, 0x0e96, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_2] = {QPI, "0a.6", "PCI_QPI_MASK_DEVICE_PORT_2", NULL, 0x0ec6, 0},
+ [PCI_QPI_MISC_DEVICE_PORT_0] = {QPI, "08.0", "PCI_QPI_MISC_DEVICE_PORT_0/1", "SBOX01FIX",0x0e80, 0},
+ [PCI_QPI_MISC_DEVICE_PORT_2] = {QPI, "0a.0", "PCI_QPI_MISC_DEVICE_PORT_2", "SBOX2FIX", 0x0ec0, 0},
+};
+
+static PciDevice haswellEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
+ [MSR_DEV] = {NONE, NULL, NULL, NULL, 0x0, 0},
+ [PCI_R3QPI_DEVICE_LINK_0] = {R3QPI, "0b.1", "PCI_R3QPI_DEVICE_LINK_0", "RBOX0", 0x2f36, 0},
+ [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "0b.2", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x2f37, 0},
+ [PCI_R2PCIE_DEVICE] = {R2PCIE, "10.1", "PCI_R2PCIE_DEVICE", "PBOX0", 0x2f34, 0},
+ [PCI_IMC_DEVICE_0_CH_0] = {IMC, "14.0", "PCI_IMC_DEVICE_0_CH_0", "MBOX0", 0x2fb4, 0},
+ [PCI_IMC_DEVICE_0_CH_1] = {IMC, "14.1", "PCI_IMC_DEVICE_0_CH_1", "MBOX1", 0x2fb5, 0},
+ [PCI_IMC_DEVICE_0_CH_2] = {IMC, "15.0", "PCI_IMC_DEVICE_0_CH_2", "MBOX2", 0x2fb0, 0},
+ [PCI_IMC_DEVICE_0_CH_3] = {IMC, "15.1", "PCI_IMC_DEVICE_0_CH_3", "MBOX3", 0x2fb1, 0},
+ [PCI_HA_DEVICE_0] = {HA, "12.1", "PCI_HA_DEVICE_0", "BBOX0", 0x2f30, 0},
+ [PCI_HA_DEVICE_1] = {HA, "12.5", "PCI_HA_DEVICE_1", "BBOX1", 0x2f38, 0},
+ [PCI_IMC_DEVICE_1_CH_0] = {IMC, "17.0", "PCI_IMC_DEVICE_1_CH_0", "MBOX4", 0x2fd4, 0},
+ [PCI_IMC_DEVICE_1_CH_1] = {IMC, "17.1", "PCI_IMC_DEVICE_1_CH_1", "MBOX5", 0x2fd5, 0},
+ [PCI_IMC_DEVICE_1_CH_2] = {IMC, "18.0", "PCI_IMC_DEVICE_1_CH_2", "MBOX6", 0x2fd0, 0},
+ [PCI_IMC_DEVICE_1_CH_3] = {IMC, "18.1", "PCI_IMC_DEVICE_1_CH_3", "MBOX7", 0x2fd1, 0},
+ [PCI_IRP_DEVICE] = {IRP, "05.6", "PCI_IRP_DEVICE", NULL, 0x2f39, 0},
+ [PCI_QPI_DEVICE_PORT_0] = {QPI, "08.2", "PCI_QPI_DEVICE_PORT_0", "SBOX0", 0x2f32, 0},
+ [PCI_QPI_DEVICE_PORT_1] = {QPI, "09.2", "PCI_QPI_DEVICE_PORT_1", "SBOX1", 0x2f33, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_0] = {QPI, "08.6", "PCI_QPI_MASK_DEVICE_PORT_0", NULL, 0x2f86, 0},
+ [PCI_QPI_MASK_DEVICE_PORT_1] = {QPI, "09.6", "PCI_QPI_MASK_DEVICE_PORT_1", NULL, 0x2f96, 0},
+ [PCI_QPI_MISC_DEVICE_PORT_0] = {QPI, "08.0", "PCI_QPI_MISC_DEVICE_PORT_0", "SBOX0FIX", 0x2f80, 0},
+ [PCI_QPI_MISC_DEVICE_PORT_1] = {QPI, "09.0", "PCI_QPI_MISC_DEVICE_PORT_1", "SBOX1FIX", 0x2f80, 0},
+};
+
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
 static int allowed_intel(uint32_t reg)
@@ -203,31 +214,7 @@ static int allowed_intel(uint32_t reg)
             (reg == 0x19C)  ||
             (reg == 0x1A2)  ||
             (reg == 0x1AD)  ||
-            (reg == 0x1A6))
-    {
-        return 1;
-    }
-    else
-    {
-        return 0;
-    }
-}
-
-static int allowed_silvermont(uint32_t reg)
-{
-    if ( ((reg & 0x0F8U) == 0x0C0U) ||
-            ((reg & 0xFF0U) == 0x180U) ||
-            ((reg & 0xF00U) == 0x300U) ||
-            ((reg & 0xF00U) == 0x600U) ||
-            ((reg & 0xF00U) == 0xC00U) ||
-            ((reg & 0xF00U) == 0xD00U) ||
-            (reg == 0x1A0)  ||
-            (reg == 0x0CE)  ||
-            (reg == 0x1AD)  ||
-            (reg == 0x19C)  ||
-            (reg == 0x1A2)  ||
-            (reg == 0x1A6) ||
-            (reg == 0x1A6) ||
+            (reg == 0x1A6)  ||
             (reg == 0x1A7))
     {
         return 1;
@@ -238,33 +225,21 @@ static int allowed_silvermont(uint32_t reg)
     }
 }
 
-static int allowed_westmereEX(uint32_t reg)
+static int allowed_sandybridge(uint32_t reg)
 {
-    if (allowed_intel(reg) == 1)
-    {
-        return 1;
-    }
-    else if ((reg & 0xF00) == 0xF00)
+    if ((allowed_intel(reg)) ||
+        (((reg & 0xF00U) == 0x600U)))
     {
         return 1;
     }
     return 0;
 }
 
-static int allowed_sandybridge(uint32_t reg)
+static int allowed_haswell(uint32_t reg)
 {
-    if ( ((reg & 0x0F8U) == 0x0C0U) ||
-            ((reg & 0xFF0U) == 0x180U) ||
-            ((reg & 0xF00U) == 0x300U) ||
-            ((reg & 0xF00U) == 0x600U) ||
-            ((reg & 0xF00U) == 0xC00U) ||
-            ((reg & 0xF00U) == 0xD00U) ||
-            (reg == 0x1A0)  ||
-            (reg == 0x0CE)  ||
-            (reg == 0x1AD)  ||
-            (reg == 0x19C)  ||
-            (reg == 0x1A2)  ||
-            (reg == 0x1A6))
+    if ((allowed_intel(reg)) ||
+        (allowed_sandybridge(reg)) ||
+        (((reg & 0xF00U) == 0x700U)))
     {
         return 1;
     }
@@ -274,22 +249,23 @@ static int allowed_sandybridge(uint32_t reg)
     }
 }
 
-static int allowed_haswell(uint32_t reg)
+
+static int allowed_silvermont(uint32_t reg)
 {
+
     if ( ((reg & 0x0F8U) == 0x0C0U) ||
             ((reg & 0xFF0U) == 0x180U) ||
             ((reg & 0xF00U) == 0x300U) ||
+            ((reg & 0xF00U) == 0x600U) ||
             ((reg & 0xF00U) == 0xC00U) ||
             ((reg & 0xF00U) == 0xD00U) ||
-            ((reg & 0xF00U) == 0xE00U) ||
-            ((reg & 0xF00U) == 0x600U) ||
-            ((reg & 0xF00U) == 0x700U) ||
             (reg == 0x1A0)  ||
             (reg == 0x0CE)  ||
+            (reg == 0x1AD)  ||
             (reg == 0x19C)  ||
             (reg == 0x1A2)  ||
-            (reg == 0x1AD)  ||
-            (reg == 0x1A6))
+            (reg == 0x1A6) ||
+            (reg == 0x1A7))
     {
         return 1;
     }
@@ -347,21 +323,16 @@ static void msr_read(AccessDataRecord * dRecord)
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
 
-    if (FD_MSR[cpu] == -2)
-    {
-        dRecord->errorcode = ERR_NODEV;
-        return;
-    }
     if (!allowed(reg))
     {
-        syslog(LOG_ERR, "attempt to read from restricted register 0x%x", reg);
+        syslog(LOG_ERR, "Attempt to read to restricted register 0x%x on core %u", reg, cpu);
         dRecord->errorcode = ERR_RESTREG;
         return;
     }
 
     if (pread(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to read data from msr device file on core %u", cpu);
+        syslog(LOG_ERR, "Failed to read data to register 0x%x on core %u", reg, cpu);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -377,22 +348,16 @@ static void msr_write(AccessDataRecord * dRecord)
 
     dRecord->errorcode = ERR_NOERROR;
 
-    if (FD_MSR[cpu] == -2)
-    {
-        dRecord->errorcode = ERR_NODEV;
-        return;
-    }
-
     if (!allowed(reg))
     {
-        syslog(LOG_ERR, "attempt to write to restricted register %x", reg);
+        syslog(LOG_ERR, "Attempt to write to restricted register 0x%x on core %u", reg, cpu);
         dRecord->errorcode = ERR_RESTREG;
         return;
     }
 
     if (pwrite(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to write data to msr device file on core %u", cpu);
+        syslog(LOG_ERR, "Failed to write data to register 0x%x on core %u", reg, cpu);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -413,26 +378,29 @@ static void pci_read(AccessDataRecord* dRecord)
         dRecord->errorcode = ERR_NODEV;
         return;
     }
-    else if ( !FD_PCI[socketId][device] )
+
+    if ( !FD_PCI[socketId][device] )
     {
         strncpy(pci_filepath, PCI_ROOT_PATH, 30);
         strncat(pci_filepath, socket_bus[socketId], 10);
-        strncat(pci_filepath, pci_DevicePath[device], 20);
-
+        strncat(pci_filepath, pci_devices[device].path, 20);
         FD_PCI[socketId][device] = open( pci_filepath, O_RDWR);
 
         if ( FD_PCI[socketId][device] < 0)
         {
-            syslog(LOG_ERR, "Failed to open device file %s on socket %u", pci_filepath, socketId);
+            syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pci_filepath,
+                    pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
+        syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
+                    pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
     }
 
-    if ( pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
+    if (FD_PCI[socketId][device] > 0 && pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to read data from pci device file on socket %u device %u",
-                socketId, device);
+        syslog(LOG_ERR, "Failed to read data from pci device file %s for device %s (%s) on socket %u",
+                pci_filepath,pci_types[pci_devices[device].type].name, pci_devices[device].name,socketId);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -450,30 +418,36 @@ static void pci_write(AccessDataRecord* dRecord)
     uint32_t data = (uint32_t) dRecord->data;
 
     dRecord->errorcode = ERR_NOERROR;
+
     if (FD_PCI[socketId][device] == -2)
     {
         dRecord->errorcode = ERR_NODEV;
         return;
     }
-    else if ( !FD_PCI[socketId][device] )
+
+    if ( !FD_PCI[socketId][device] )
     {
         strncpy(pci_filepath, PCI_ROOT_PATH, 30);
         strncat(pci_filepath, socket_bus[socketId], 10);
-        strncat(pci_filepath, pci_DevicePath[device], 20);
+        strncat(pci_filepath, pci_devices[device].path, 20);
 
         FD_PCI[socketId][device] = open( pci_filepath, O_RDWR);
 
         if ( FD_PCI[socketId][device] < 0)
         {
-            syslog(LOG_ERR, "Failed to open device file %s on socket %u", pci_filepath, socketId);
+            syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pci_filepath,
+                        pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
+        syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
+                    pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
     }
 
-    if (pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
+    if (FD_PCI[socketId][device] > 0 && pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
     {
-        syslog(LOG_ERR, "Failed to write data to pci device file on socket %u", socketId);
+        syslog(LOG_ERR, "Failed to write data to pci device file %s for device %s (%s) on socket %u",pci_filepath,
+                pci_types[pci_devices[device].type].name, pci_devices[device].name, socketId);
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
@@ -496,6 +470,13 @@ static void stop_daemon(void)
 {
     kill_client();
     syslog(LOG_NOTICE, "daemon exiting");
+    for (int i=0;i<MAX_NUM_NODES;i++)
+    {
+        if (socket_bus[i] != NULL)
+        {
+            free(socket_bus[i]);
+        }
+    }
 
     if (sockfd != -1)
     {
@@ -507,6 +488,41 @@ static void stop_daemon(void)
     exit(EXIT_SUCCESS);
 }
 
+int getBusFromSocket(const uint32_t socket)
+{
+    int cur_bus = 0;
+    uint32_t cur_socket = 0;
+    char pci_filepath[1024];
+    int fp;
+    int ret = 0;
+    while(cur_socket <= socket)
+    {
+        sprintf(pci_filepath, "%s%02x/05.0", PCI_ROOT_PATH, cur_bus);
+        fp = open(pci_filepath, O_RDONLY);
+        if (fp < 0)
+        {
+            return -1;
+        }
+        uint32_t cpubusno = 0;
+        ret = pread(fp, &cpubusno, sizeof(uint32_t), 0x108);
+        if (ret != sizeof(uint32_t))
+        {
+            close(fp);
+            return -1;
+        }
+        cur_bus = (cpubusno >> 8) & 0x0ff;
+        close(fp);
+        if(socket == cur_socket)
+            return cur_bus;
+        ++cur_socket;
+        ++cur_bus;
+        if(cur_bus > 0x0ff)
+           return -1;
+    }
+
+    return -1;
+}
+
 static void Signal_Handler(int sig)
 {
     if (sig == SIGPIPE)
@@ -516,7 +532,7 @@ static void Signal_Handler(int sig)
     }
 
     /* For SIGALRM we just return - we're just here to create a EINTR */
-    if ((sig == SIGTERM))
+    if (sig == SIGTERM)
     {
         stop_daemon();
     }
@@ -543,7 +559,7 @@ static void daemonize(int* parentPid)
     /* If we got a good PID, then we can exit the parent process. */
     if (pid > 0)
     {
-        exit(ERR_NOERROR);
+        exit(EXIT_SUCCESS);
     }
 
     /* At this point we are executing as the child process */
@@ -585,90 +601,88 @@ int main(void)
     mode_t oldumask;
     uint32_t numHWThreads = sysconf(_SC_NPROCESSORS_CONF);
     uint32_t model;
-    int isIntel = 1;
+#ifdef REVERSE_HASWELL_PCI_SOCKETS
+    char** socket_bus_copy;
+#endif
+
+
+    openlog(ident, 0, LOG_USER);
 
     if (!lock_check())
     {
-        fprintf(stderr,"Access to performance counters is locked. Exiting!\n");
-        exit(EXIT_FAILURE);
+        syslog(LOG_ERR,"Access to performance counters is locked.\n");
+        stop_daemon();
     }
 
-    for ( uint32_t i=0; i < numHWThreads; i++ )
-    {
-        FD_MSR[i] = -1;
-    }
-
-    uint32_t  eax = 0x00;
-    uint32_t  ebx = 0x00;
-    
-    CPUID;
-    if (ebx == 0x68747541U)
-    {
-        isIntel = 0;
-    }
-
-    eax = 0x01;
-    CPUID;
-    uint32_t family = ((eax >> 8) & 0xFU) + ((eax >> 20) & 0xFFU);
-    model  = (((eax >> 16) & 0xFU) << 4) + ((eax >> 4) & 0xFU);
-
-    switch (family)
-    {
-        case P6_FAMILY:
-            allowed = allowed_intel;
-
-            if (isIntel && ((model == SANDYBRIDGE)    ||
-                            (model == SANDYBRIDGE_EP) ||
-                            (model == IVYBRIDGE)      ||
-                            (model == IVYBRIDGE_EP) ))
-            {
-                allowed = allowed_sandybridge;
-                isPCIUncore = 1;
-            }
-            else if (isIntel && ((model == HASWELL)    ||
-                                 (model == HASWELL_M1) ||
-                                 (model == HASWELL_M2) ||
-                                 (model == HASWELL_EX)))
-            {
-                allowed = allowed_haswell;
-            }
-            else if (isIntel && (model == ATOM_SILVERMONT))
-            {
-                allowed = allowed_silvermont;
-            }
-            else if (isIntel && (model == WESTMERE_EX))
-            {
-                allowed = allowed_westmereEX;
-            }
-            break;
-        case K8_FAMILY:
-        case K10_FAMILY:
-            if (!isIntel) 
-            {
-                allowed = allowed_amd;
-            }
-            break;
-        case K15_FAMILY:
-            if (!isIntel) 
-            {
-                allowed = allowed_amd15;
-            }
-            break;
-        case K16_FAMILY:
-            if (!isIntel) 
-            {
-                allowed = allowed_amd16;
-            }
-            break;
-        default:
-            fprintf(stderr, "ERROR - [%s:%d] - Unsupported processor. Exiting!\n",
-                    __FILE__, __LINE__);
-            exit(EXIT_FAILURE);
-    }
-
-    openlog(ident, 0, LOG_USER);
     daemonize(&pid);
 
+    {
+        uint32_t  eax = 0x00;
+        uint32_t  ebx = 0x00;
+        /*int isIntel = 1;
+        CPUID;
+        if (ebx == 0x68747541U)
+        {
+            isIntel = 0;
+        }*/
+
+        eax = 0x01;
+        CPUID;
+        uint32_t family = ((eax >> 8) & 0xFU) + ((eax >> 20) & 0xFFU);
+        model  = (((eax >> 16) & 0xFU) << 4) + ((eax >> 4) & 0xFU);
+
+        switch (family)
+        {
+            case P6_FAMILY:
+                allowed = allowed_intel;
+
+                if ((model == SANDYBRIDGE) || (model == IVYBRIDGE))
+                {
+                    allowed = allowed_sandybridge;
+                }
+                else if ((model == SANDYBRIDGE_EP) || (model == IVYBRIDGE_EP))
+                {
+                    allowed = allowed_sandybridge;
+                    isPCIUncore = 1;
+                }
+                else if ((model == HASWELL) ||
+                         (model == BROADWELL) ||
+                         (model == BROADWELL_D) ||
+                         (model == BROADWELL_E))
+                {
+                    allowed = allowed_haswell;
+                }
+                else if (model == HASWELL_EP)
+                {
+                    isPCIUncore = 1;
+                    allowed = allowed_haswell;
+                }
+                else if ((model == ATOM_SILVERMONT_C) ||
+                         (model == ATOM_SILVERMONT_E) ||
+                         (model == ATOM_SILVERMONT_Z1) ||
+                         (model == ATOM_SILVERMONT_Z2) ||
+                         (model == ATOM_SILVERMONT_F))
+                {
+                    allowed = allowed_silvermont;
+                }
+                break;
+            case K8_FAMILY:
+            case K10_FAMILY:
+                allowed = allowed_amd;
+                break;
+            case K15_FAMILY:
+                allowed = allowed_amd15;
+                break;
+            case K16_FAMILY:
+                allowed = allowed_amd16;
+            break;
+            default:
+                syslog(LOG_ERR, "ERROR - [%s:%d] - Unsupported processor. Exiting!  \n",
+                        __FILE__, __LINE__);
+                exit(EXIT_FAILURE);
+        }
+    }
+
     /* setup filename for socket */
     filepath = (char*) calloc(sizeof(addr1.sun_path), 1);
     snprintf(filepath, sizeof(addr1.sun_path), "/tmp/likwid-%d", pid);
@@ -691,10 +705,6 @@ int main(void)
     EXIT_IF_ERROR(listen(sockfd, 1), listen failed);
     EXIT_IF_ERROR(chmod(filepath, S_IRUSR|S_IWUSR), chmod failed);
 
-    /* Restore the old umask and fs ids. */
-    (void) umask(oldumask);
-    CHECK_ERROR(setfsuid(geteuid()), setfsuid failed);
-
     socklen = sizeof(addr1);
 
     { /* Init signal handler */
@@ -730,6 +740,10 @@ int main(void)
     CHECK_ERROR(unlink(filepath), unlink of socket failed);
     syslog(LOG_NOTICE, "daemon accepted client");
 
+    /* Restore the old umask and fs ids. */
+    (void) umask(oldumask);
+    CHECK_ERROR(setfsuid(geteuid()), setfsuid failed);
+
     {
         char* msr_file_name = (char*) malloc(MAX_PATH_LENGTH * sizeof(char));
 
@@ -737,75 +751,66 @@ int main(void)
          * NOTICE: This assumes consecutive processor Ids! */
         for ( uint32_t i=0; i < numHWThreads; i++ )
         {
-#ifdef __MIC
-            sprintf(msr_file_name,"/dev/msr%d",i);
-            if (access(msr_file_name, F_OK))
-            {
-                sprintf(msr_file_name,"/dev/cpu/%d/msr",i);
-            }
-#else
             sprintf(msr_file_name,"/dev/cpu/%d/msr",i);
-#endif
             FD_MSR[i] = open(msr_file_name, O_RDWR);
 
             if ( FD_MSR[i] < 0 )
             {
-                syslog(LOG_ERR, "Failed to open device file %s.",msr_file_name);
-                FD_MSR[i] = -2;
+                syslog(LOG_ERR, "Failed to open device file %s: %s, trying /dev/msr%d", msr_file_name, strerror(errno), i);
+                sprintf(msr_file_name,"/dev/msr%d",i);
+                FD_MSR[i] = open(msr_file_name, O_RDWR);
+                if ( FD_MSR[i] < 0 )
+                {
+                    syslog(LOG_ERR, "Failed to open device file %s: %s.", msr_file_name, strerror(errno));
+                }
             }
         }
 
         free(msr_file_name);
-
         if (isPCIUncore)
         {
+            int cntr = 0;
+            int socket_count = 0;
+            if (model == SANDYBRIDGE_EP)
+            {
+                //testDevice = 0x80863c44;
+                pci_devices = sandybridgeEP_pci_devices;
+            }
+            else if (model == IVYBRIDGE_EP)
+            {
+                //testDevice = 0x80860e36;
+                pci_devices = ivybridgeEP_pci_devices;
+            }
+            else if (model == HASWELL_EP)
+            {
+                //testDevice = 0x80862f30;
+                pci_devices = haswellEP_pci_devices;
+            }
+            else
+            {
+                //testDevice = 0;
+                syslog(LOG_NOTICE, "PCI Uncore not supported on this system");
+            }
+
             for (int j=0; j<MAX_NUM_NODES; j++)
             {
                 socket_bus[j] = "N-A";
-                for (int i=0; i<MAX_NUM_DEVICES; i++)
+                for (int i=0; i<MAX_NUM_PCI_DEVICES; i++)
                 {
                     FD_PCI[j][i] = -2;
                 }
             }
 
             /* determine PCI-BUSID mapping ... */
-            FILE *fptr;
-            char buf[1024];
-            uint32_t testDevice;
-            uint32_t sbus, sdevfn, svend;
-            int cntr = 0;
-            int socket_count = 0;
-
-            if (model == SANDYBRIDGE_EP)
+            int sbus = -1;
+            cntr = 0;
+            sbus = getBusFromSocket(cntr);
+            while (sbus != -1)
             {
-                testDevice = 0x80863c44;
-            }
-            else if (model == IVYBRIDGE_EP)
-            {
-                testDevice = 0x80860e36;
-            }
-            else
-            {
-                testDevice = 0;
-                syslog(LOG_NOTICE, "PCI Uncore not supported on this system");
-            }
-
-            if ( ((fptr = fopen("/proc/bus/pci/devices", "r")) == NULL) || !testDevice)
-            {
-                syslog(LOG_NOTICE, "Unable to open /proc/bus/pci/devices");
-            }
-            else
-            {
-                while( fgets(buf, sizeof(buf)-1, fptr) )
-                {
-                    if ( sscanf(buf, "%2x%2x %8x", &sbus, &sdevfn, &svend) == 3 &&
-                            svend == testDevice )
-                    {
-                        socket_bus[cntr] = (char*)malloc(4);
-                        sprintf(socket_bus[cntr++], "%02x/", sbus);
-                    }
-                }
-                fclose(fptr);
+                socket_bus[cntr] = (char*)malloc(4);
+                sprintf(socket_bus[cntr], "%02x/", sbus);
+                cntr++;
+                sbus = getBusFromSocket(cntr);
             }
 
             if ( cntr == 0 )
@@ -815,20 +820,25 @@ int main(void)
             else
             {
                 socket_count = cntr;
-
+                int fd;
                 for (int j=0; j<socket_count; j++)
                 {
-                    for (int i=0; i<MAX_NUM_DEVICES; i++)
+                    for (int i=1; i<MAX_NUM_PCI_DEVICES; i++)
                     {
-                        sprintf(pci_filepath, "%s%s%s",PCI_ROOT_PATH,socket_bus[j],pci_DevicePath[i]);
-
-                        if (!access(pci_filepath,F_OK))
+                        if (pci_devices[i].path)
                         {
-                            FD_PCI[j][i] = 0;
-                        }
-                        else
-                        {
-                            syslog(LOG_NOTICE, "Device %s not found, excluded it from device list\n",pci_filepath);
+                            sprintf(pci_filepath, "%s%s%s", PCI_ROOT_PATH, socket_bus[j], pci_devices[i].path);
+                            fd = open(pci_filepath, O_RDWR);
+                            if (fd > 0)
+                            {
+                                FD_PCI[j][i] = 0;
+                                pci_devices[i].online = 1;
+                                close(fd);
+                            }
+                            else if (j==0)
+                            {
+                                syslog(LOG_NOTICE, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices[i].name,j, pci_filepath, strerror(errno));
+                            }
                         }
                     }
                 }
@@ -846,7 +856,7 @@ int main(void)
                     __FILE__, __LINE__, strerror(errno));
             stop_daemon();
         }
-        else if (ret == 0)
+        else if ((ret == 0) && (dRecord.type != DAEMON_EXIT))
         {
             syslog(LOG_ERR, "ERROR - [%s:%d] zero read", __FILE__, __LINE__);
             stop_daemon();
