@@ -3,16 +3,16 @@
  *
  *      Filename:  setFreq.c
  *
- *      Description:  Wrapper for accessing setfreq kernel FS files
+ *      Description:  Implementation of frequency daemon
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Authors:  Michael Meier, michael.meier@rrze.fau.de
- *                Jan Treibig (jt), jan.treibig@gmail.com
+ *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -28,11 +28,12 @@
  *
  * =======================================================================================
  */
-
+/* #####   HEADER FILE INCLUDES   ######################################### */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+/* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 static int get_numCPUs()
 {
     int cpucount = 0;
@@ -51,6 +52,7 @@ static int get_numCPUs()
     return cpucount;
 }
 
+/* #####  MAIN FUNCTION DEFINITION   ################## */
 int main (int argn, char** argv)
 {
     int cpuid;
@@ -59,11 +61,12 @@ int main (int argn, char** argv)
     char* gov;
     char* gpath = malloc(100);
     char* fpath = malloc(100);
-    FILE* f;
 
     if (argn < 3 || argn > 4)
     {
         fprintf(stderr, "Usage: %s <processorID> <frequency> [<governor>] \n",argv[0]);
+        free(gpath);
+        free(fpath);
         exit(EXIT_FAILURE);
     }
 
@@ -72,58 +75,72 @@ int main (int argn, char** argv)
     if (cpuid < 0 || cpuid > numCPUs)
     {
         fprintf(stderr, "CPU %d not a valid CPU ID. Range from 0 to %d.\n",cpuid,numCPUs);
+        free(gpath);
+        free(fpath);
         exit(EXIT_FAILURE);
     }
     freq  = atoi(argv[2]);
-    if (freq < 0)
+    if (freq <= 0)
     {
         fprintf(stderr, "Frequency must be greater than 0.\n");
+        free(gpath);
+        free(fpath);
         exit(EXIT_FAILURE);
     }
-    snprintf(gpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
-    snprintf(fpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_setspeed", cpuid);
 
     if (argn == 4)
     {
         gov = argv[3];
 
-        if ((strncmp(gov,"ondemand",12)) && (strncmp(gov,"performance",12)))
-        {
+        if ((strncmp(gov,"ondemand",8)) &&
+            (strncmp(gov,"performance",11)) &&
+            (strncmp(gov,"conservative",12)) &&
+            (strncmp(gov,"powersave",9))) {
             fprintf(stderr, "Invalid governor %s!\n",gov);
+            free(gpath);
+            free(fpath);
             return (EXIT_FAILURE);
         }
+        snprintf(gpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
 
-        f = fopen(gpath, "w");
-        if (f == NULL)
-        {
+        FILE* f = fopen(gpath, "w");
+        if (f == NULL) {
             fprintf(stderr, "Unable to open path for writing\n");
+            free(gpath);
+            free(fpath);
             return (EXIT_FAILURE);
         }
         fprintf(f,"%s",gov);
         fclose(f);
+        free(gpath);
+        free(fpath);
         return(EXIT_SUCCESS);
     }
-    else
-    {
-        f = fopen(gpath, "w");
-        if (f == NULL)
-        {
-            fprintf(stderr, "Unable to open path for writing\n");
-            return (EXIT_FAILURE);
-        }
-        fprintf(f,"userspace");
-        fclose(f);
+
+    snprintf(gpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_governor", cpuid);
+    snprintf(fpath, 60, "/sys/devices/system/cpu/cpu%d/cpufreq/scaling_setspeed", cpuid);
+
+    FILE* f = fopen(gpath, "w");
+    if (f == NULL) {
+        fprintf(stderr, "Unable to open path for writing\n");
+        free(gpath);
+        free(fpath);
+        return (EXIT_FAILURE);
     }
+    fprintf(f,"userspace");
+    fclose(f);
 
     f = fopen(fpath, "w");
-    if (f == NULL)
-    {
+    if (f == NULL) {
         fprintf(stderr, "Unable to open path for writing\n");
+        free(gpath);
+        free(fpath);
         return (EXIT_FAILURE);
     }
     fprintf(f,"%d",freq);
     fclose(f);
-
+    free(gpath);
+    free(fpath);
     return(EXIT_SUCCESS);
 }
 
