@@ -5,16 +5,16 @@
  *
  *      Description:  Implementation of client to the access daemon.
  *                   Provides API to read and write values to MSR or
- *                   PCI Cfg Adresses. This module is used by the
+ *                   PCI Cfg Adresses. This module is used by the 
  *                   msr and pci modules.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -50,6 +50,8 @@
 #include <error.h>
 #include <cpuid.h>
 #include <accessClient.h>
+#include <perfmon.h>
+#include <configuration.h>
 
 int accessClient_mode = ACCESSMODE;
 
@@ -60,7 +62,8 @@ int accessClient_mode = ACCESSMODE;
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
-static char* accessClient_strerror(AccessErrorType det)
+static char*
+accessClient_strerror(AccessErrorType det)
 {
     switch (det)
     {
@@ -70,14 +73,29 @@ static char* accessClient_strerror(AccessErrorType det)
         case ERR_OPENFAIL:   return "failed to open device file";
         case ERR_RWFAIL:     return "failed to read/write register";
         case ERR_DAEMONBUSY: return "daemon already has a same/higher priority client";
-        case ERR_LOCKED:     return "access to HPM is locked";
-        case ERR_UNSUPPORTED: return "unsupported processor";
-        case ERR_NODEV:      return "no such device";
+        case ERR_NODEV:      return "no such pci device";
         default:             return "UNKNOWN errorcode";
     }
 }
 
-static int startDaemon(void)
+static int
+accessClient_errno(AccessErrorType det)
+{
+    switch (det)
+    {
+        case ERR_NOERROR:    return 0;
+        case ERR_UNKNOWN:    return -EFAULT;
+        case ERR_RESTREG:    return -EPERM;
+        case ERR_OPENFAIL:   return -ENXIO;
+        case ERR_RWFAIL:     return -EIO;
+        case ERR_DAEMONBUSY: return -EBUSY;
+        case ERR_NODEV:      return -ENODEV;
+        default:             return -EFAULT;
+    }
+}
+
+static int
+startDaemon(void)
 {
     /* Check the function of the daemon here */
     char* filepath;
@@ -91,26 +109,36 @@ static int startDaemon(void)
     int timeout = 1000;
     int socket_fd = -1;
 
-    if (accessClient_mode == DAEMON_AM_ACCESS_D)
+    if (accessClient_mode == ACCESSMODE_DIRECT)
+    {
+        return 0;
+    }
+
+    if (config.daemonPath != NULL)
+    {
+        strcpy(exeprog, config.daemonPath);
+    }
+
+    if (access(exeprog, X_OK))
+    {
+        ERROR_PRINT(Failed to find the daemon '%s'\n, exeprog);
+        exit(EXIT_FAILURE);
+    }
+
+    if (accessClient_mode == ACCESSMODE_DAEMON)
     {
-        if (access(exeprog, F_OK))
-        {
-            fprintf(stderr, "Daemon '%s' cannot be found\n", exeprog);
-            exit(EXIT_FAILURE);
-        }
-        if (access(exeprog, X_OK))
-        {
-            fprintf(stderr, "Daemon '%s' not executable\n", exeprog);
-            exit(EXIT_FAILURE);
-        }
         pid = fork();
 
         if (pid == 0)
         {
             ret = execve (exeprog, newargv, newenv);
-            ERRNO_PRINT;
-            fprintf(stderr, "Failed to execute the daemon '%s' (see error above)\n", exeprog);
-            exit(EXIT_FAILURE);
+
+            if (ret < 0)
+            {
+                //ERRNO_PRINT;
+                ERROR_PRINT(Failed to execute the daemon '%s'\n, exeprog);
+                exit(EXIT_FAILURE);
+            }
         }
         else if (pid < 0)
         {
@@ -124,9 +152,10 @@ static int startDaemon(void)
     address_length = sizeof(address);
     snprintf(address.sun_path, sizeof(address.sun_path), "/tmp/likwid-%d", pid);
     filepath = strdup(address.sun_path);
-    DEBUG_PRINT(0, "%ssocket pathname is %s\n",
-            ((accessClient_mode == DAEMON_AM_ACCESS_D) ? "Generated " : ""),
-            filepath);
+    if (accessClient_mode == ACCESSMODE_DAEMON)
+    {
+        DEBUG_PRINT(DEBUGLEV_INFO, Socket pathname is %s, filepath);
+    }
 
     while (timeout > 0)
     {
@@ -140,9 +169,9 @@ static int startDaemon(void)
         }
 
         timeout--;
-        DEBUG_PRINT(1, "%s\n", "Still waiting for socket...");
+        DEBUG_PRINT(DEBUGLEV_INFO, Still waiting for socket %s ..., filepath);
     }
-
+    
     if (timeout <= 0)
     {
         ERRNO_PRINT;  /* should hopefully still work, as we make no syscalls in between. */
@@ -151,12 +180,9 @@ static int startDaemon(void)
         fprintf(stderr, "Consult the error message above this to find out why.\n");
         fprintf(stderr, "If the error is 'no such file or directoy', \
                 it usually means that likwid-accessD just failed to start.\n");
-        fprintf(stderr, "In case the daemon itself output an error', \
-                ignore this.\n");
         exit(EXIT_FAILURE);
     }
-
-    DEBUG_PRINT(0, "%s\n", "Successfully opened socket to daemon.");
+    DEBUG_PRINT(DEBUGLEV_INFO, Successfully opened socket %s to daemon, filepath);
     free(filepath);
 
     return socket_fd;
@@ -164,26 +190,32 @@ static int startDaemon(void)
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void accessClient_setaccessmode(int mode)
+void 
+accessClient_setaccessmode(int mode)
 {
-    if ((accessClient_mode > DAEMON_AM_ACCESS_D) || (accessClient_mode < DAEMON_AM_DIRECT))
+    if ((accessClient_mode > ACCESSMODE_DAEMON) || (accessClient_mode < ACCESSMODE_DIRECT))
     {
-        fprintf(stderr, "Invalid accessmode %d\n", accessClient_mode);
+        ERROR_PRINT(Invalid accessmode %d, accessClient_mode);
         exit(EXIT_FAILURE);
     }
-
     accessClient_mode = mode;
 }
 
-void accessClient_init(int* socket_fd)
+void 
+accessClient_init(int* socket_fd)
 {
-    if ((accessClient_mode == DAEMON_AM_ACCESS_D))
+    if (config.daemonMode != -1)
+    {
+        accessClient_mode = config.daemonMode;
+    }
+    if ((accessClient_mode == ACCESSMODE_DAEMON) && (*socket_fd == -1))
     {
         (*socket_fd) = startDaemon();
     }
 }
 
-void accessClient_finalize(int socket_fd)
+void 
+accessClient_finalize(int socket_fd)
 {
     if ( socket_fd != -1 )
     { /* Only if a socket is actually open */
@@ -195,11 +227,13 @@ void accessClient_finalize(int socket_fd)
 }
 
 
-uint64_t accessClient_read(
+int
+accessClient_read(
         int socket_fd,
         const int cpu,
         const int device,
-        uint32_t reg)
+        uint32_t reg,
+        uint64_t *result)
 {
     AccessDataRecord data;
 
@@ -214,16 +248,16 @@ uint64_t accessClient_read(
 
     if (data.errorcode != ERR_NOERROR)
     {
-        fprintf(stderr, "Failed to read data through daemon: "
-                "daemon returned error %d '%s' for cpu %d reg 0x%x\n",
-                data.errorcode, accessClient_strerror(data.errorcode), cpu, reg);
-        //exit(EXIT_FAILURE);
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Got error '%s' from access daemon reading reg 0x%X at CPU %d, accessClient_strerror(data.errorcode), data.reg, data.cpu);
+        *result = 0;
+        return accessClient_errno(data.errorcode);
     }
-
-    return data.data;
+    *result = data.data;
+    return 0;
 }
 
-void accessClient_write(
+int 
+accessClient_write(
         int socket_fd,
         const int cpu,
         const int device,
@@ -242,16 +276,11 @@ void accessClient_write(
 
     if (data.errorcode != ERR_NOERROR)
     {
-        fprintf(stderr, "Failed to write data through daemon: "
-                "daemon returned error %d '%s' for cpu %d reg 0x%x\n",
-                data.errorcode, accessClient_strerror(data.errorcode), cpu, reg);
-        //exit(EXIT_FAILURE);
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, Got error '%s' from access daemon writing reg 0x%X at CPU %d, accessClient_strerror(data.errorcode), data.reg, data.cpu);
+        return accessClient_errno(data.errorcode);
     }
 
-    if (data.data != 0x00ULL)
-    {
-        ERROR_PLAIN_PRINT(daemon write failed);
-    }
+    return 0;
 }
 
 
