@@ -9,13 +9,13 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,8 +39,8 @@
 #include <stdint.h>
 
 #include <types.h>
-#include <msr.h>
-#include <cpuid.h>
+#include <access.h>
+#include <topology.h>
 #include <registers.h>
 #include <textcolor.h>
 #include <cpuFeatures.h>
@@ -73,7 +73,9 @@ CpuFeatureFlags cpuFeatureFlags;
 void
 cpuFeatures_init(int cpu)
 {
-    uint64_t flags = msr_read(cpu, MSR_IA32_MISC_ENABLE);
+    int ret;
+    uint64_t flags;
+    ret = HPMread(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, &flags);
 
     TEST_FLAG(fastStrings,0);
     TEST_FLAG(thermalControl,3);
@@ -120,7 +122,9 @@ cpuFeatures_init(int cpu)
 void
 cpuFeatures_print(int cpu)
 {
-    uint64_t flags = msr_read(cpu, MSR_IA32_MISC_ENABLE);
+    int ret;
+    uint64_t flags;
+    ret = HPMread(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, &flags);
 
     printf(HLINE);
     printf("Fast-Strings: \t\t\t");
@@ -154,7 +158,7 @@ cpuFeatures_print(int cpu)
     }
     printf("Branch Trace Storage: \t\t");
 
-    if (flags & (1ULL<<11))
+    if (flags & (1ULL<<11)) 
     {
         PRINT_VALUE(RED,notsupported);
     }
@@ -164,7 +168,7 @@ cpuFeatures_print(int cpu)
     }
 
     printf("PEBS: \t\t\t\t");
-    if (flags & (1ULL<<12))
+    if (flags & (1ULL<<12)) 
     {
         PRINT_VALUE(RED,notsupported);
     }
@@ -174,7 +178,7 @@ cpuFeatures_print(int cpu)
     }
 
     printf("Intel Enhanced SpeedStep: \t");
-    if (flags & (1ULL<<16))
+    if (flags & (1ULL<<16)) 
     {
         PRINT_VALUE(GREEN,enabled);
     }
@@ -184,7 +188,7 @@ cpuFeatures_print(int cpu)
     }
 
     printf("MONITOR/MWAIT: \t\t\t");
-    if (flags & (1ULL<<18))
+    if (flags & (1ULL<<18)) 
     {
         PRINT_VALUE(GREEN,supported);
     }
@@ -194,7 +198,7 @@ cpuFeatures_print(int cpu)
     }
 
     printf("Limit CPUID Maxval: \t\t");
-    if (flags & (1ULL<<22))
+    if (flags & (1ULL<<22)) 
     {
         PRINT_VALUE(RED,enabled);
     }
@@ -204,7 +208,7 @@ cpuFeatures_print(int cpu)
     }
 
     printf("XD Bit Disable: \t\t");
-    if (flags & (1ULL<<34))
+    if (flags & (1ULL<<34)) 
     {
         PRINT_VALUE(RED,disabled);
     }
@@ -212,53 +216,45 @@ cpuFeatures_print(int cpu)
     {
         PRINT_VALUE(GREEN,enabled);
     }
-    if ((cpuid_info.model == NEHALEM) ||
-            (cpuid_info.model == NEHALEM_BLOOMFIELD) ||
-            (cpuid_info.model == NEHALEM_LYNNFIELD) ||
-            (cpuid_info.model == NEHALEM_WESTMERE) ||
-            (cpuid_info.model == NEHALEM_WESTMERE_M) ||
-            (cpuid_info.model == NEHALEM_EX) ||
-            (cpuid_info.model == CORE2_45) ||
-            (cpuid_info.model == CORE2_65))
+
+    printf("IP Prefetcher: \t\t\t");
+    if (flags & (1ULL<<39)) 
     {
-        printf("IP Prefetcher: \t\t\t");
-        if (flags & (1ULL<<39))
-        {
-            PRINT_VALUE(RED,disabled);
-        }
-        else
-        {
-            PRINT_VALUE(GREEN,enabled);
-        }
+        PRINT_VALUE(RED,disabled);
+    }
+    else
+    {
+        PRINT_VALUE(GREEN,enabled);
+    }
 
-        printf("Hardware Prefetcher: \t\t");
-        if (flags & (1ULL<<9))
-        {
-            PRINT_VALUE(RED,disabled);
-        }
-        else
-        {
-            PRINT_VALUE(GREEN,enabled);
-        }
-        printf("Adjacent Cache Line Prefetch: \t");
-        if (flags & (1ULL<<19))
-        {
-            PRINT_VALUE(RED,disabled);
-        }
-        else
-        {
-            PRINT_VALUE(GREEN,enabled);
-        }
+    printf("Hardware Prefetcher: \t\t");
+    if (flags & (1ULL<<9)) 
+    {
+        PRINT_VALUE(RED,disabled);
+    }
+    else
+    {
+        PRINT_VALUE(GREEN,enabled);
+    }
 
-        printf("DCU Prefetcher: \t\t");
-        if (flags & (1ULL<<37))
-        {
-            PRINT_VALUE(RED,disabled);
-        }
-        else
-        {
-            PRINT_VALUE(GREEN,enabled);
-        }
+    printf("Adjacent Cache Line Prefetch: \t");
+    if (flags & (1ULL<<19)) 
+    {
+        PRINT_VALUE(RED,disabled);
+    }
+    else
+    {
+        PRINT_VALUE(GREEN,enabled);
+    }
+
+    printf("DCU Prefetcher: \t\t");
+    if (flags & (1ULL<<37)) 
+    {
+        PRINT_VALUE(RED,disabled);
+    }
+    else
+    {
+        PRINT_VALUE(GREEN,enabled);
     }
 
     if ((cpuid_info.model == NEHALEM) ||
@@ -268,12 +264,12 @@ cpuFeatures_print(int cpu)
             (cpuid_info.model == NEHALEM_WESTMERE_M) ||
             (cpuid_info.model == NEHALEM_EX))
     {
-        printf("Intel Turbo Mode: \t\t");
-        if (flags & (1ULL<<38))
+        printf("Intel Turbo Mode: \t");
+        if (flags & (1ULL<<38)) 
         {
             PRINT_VALUE(RED,disabled);
         }
-        else
+        else 
         {
             PRINT_VALUE(GREEN,enabled);
         }
@@ -283,11 +279,11 @@ cpuFeatures_print(int cpu)
     {
 
         printf("Intel Dynamic Acceleration: \t");
-        if (flags & (1ULL<<38))
+        if (flags & (1ULL<<38)) 
         {
             PRINT_VALUE(RED,disabled);
         }
-        else
+        else 
         {
             PRINT_VALUE(GREEN,enabled);
         }
@@ -296,104 +292,82 @@ cpuFeatures_print(int cpu)
     printf(HLINE);
 }
 
-void
+void 
 cpuFeatures_enable(int cpu, CpuFeature type)
 {
-    if ((cpuid_info.model == NEHALEM) ||
-            (cpuid_info.model == NEHALEM_BLOOMFIELD) ||
-            (cpuid_info.model == NEHALEM_LYNNFIELD) ||
-            (cpuid_info.model == NEHALEM_WESTMERE) ||
-            (cpuid_info.model == NEHALEM_WESTMERE_M) ||
-            (cpuid_info.model == NEHALEM_EX) ||
-            (cpuid_info.model == CORE2_45) ||
-            (cpuid_info.model == CORE2_65))
+    int ret;
+    uint64_t flags; 
+    ret = HPMread(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, &flags);
+
+    switch ( type )
     {
-        uint64_t flags = msr_read(cpu, MSR_IA32_MISC_ENABLE);
-        switch ( type )
-        {
-            case HW_PREFETCHER:
-                printf("HW_PREFETCHER:\t");
-                flags &= ~(1ULL<<9);
-                break;
+        case HW_PREFETCHER:
+            printf("HW_PREFETCHER:\t");
+            flags &= ~(1ULL<<9);
+            break;
 
-            case CL_PREFETCHER:
-                printf("CL_PREFETCHER:\t");
-                flags &= ~(1ULL<<19);
-                break;
+        case CL_PREFETCHER:
+            printf("CL_PREFETCHER:\t");
+            flags &= ~(1ULL<<19);
+            break;
 
-            case DCU_PREFETCHER:
-                printf("DCU_PREFETCHER:\t");
-                flags &= ~(1ULL<<37);
-                break;
+        case DCU_PREFETCHER:
+            printf("DCU_PREFETCHER:\t");
+            flags &= ~(1ULL<<37);
+            break;
 
-            case IP_PREFETCHER:
-                printf("IP_PREFETCHER:\t");
-                flags &= ~(1ULL<<39);
-                break;
+        case IP_PREFETCHER:
+            printf("IP_PREFETCHER:\t");
+            flags &= ~(1ULL<<39);
+            break;
 
-            default:
-                printf("ERROR: CpuFeature not supported!\n");
-                break;
-        }
-        PRINT_VALUE(GREEN,enabled);
-        printf("\n");
-        msr_write(cpu, MSR_IA32_MISC_ENABLE, flags);
-    }
-    else
-    {
-        printf("ERROR: Architecture does not support the manipulation of prefetchers\n");
+        default:
+            printf("ERROR: CpuFeature not supported!\n");
+            break;
     }
+    PRINT_VALUE(GREEN,enabled);
+    printf("\n");
+
+    HPMwrite(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, flags);
 }
 
 
 void
 cpuFeatures_disable(int cpu, CpuFeature type)
 {
-    if ((cpuid_info.model == NEHALEM) ||
-            (cpuid_info.model == NEHALEM_BLOOMFIELD) ||
-            (cpuid_info.model == NEHALEM_LYNNFIELD) ||
-            (cpuid_info.model == NEHALEM_WESTMERE) ||
-            (cpuid_info.model == NEHALEM_WESTMERE_M) ||
-            (cpuid_info.model == NEHALEM_EX) ||
-            (cpuid_info.model == CORE2_45) ||
-            (cpuid_info.model == CORE2_65))
+    int ret;
+    uint64_t flags;
+    ret = HPMread(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, &flags);
+
+    switch ( type ) 
     {
-        uint64_t flags = msr_read(cpu, MSR_IA32_MISC_ENABLE);
+        case HW_PREFETCHER:
+            printf("HW_PREFETCHER:\t");
+            flags |= (1ULL<<9);
+            break;
 
-        switch ( type )
-        {
-            case HW_PREFETCHER:
-                printf("HW_PREFETCHER:\t");
-                flags |= (1ULL<<9);
-                break;
+        case CL_PREFETCHER:
+            printf("CL_PREFETCHER:\t");
+            flags |= (1ULL<<19);
+            break;
 
-            case CL_PREFETCHER:
-                printf("CL_PREFETCHER:\t");
-                flags |= (1ULL<<19);
-                break;
+        case DCU_PREFETCHER:
+            printf("DCU_PREFETCHER:\t");
+            flags |= (1ULL<<37);
+            break;
 
-            case DCU_PREFETCHER:
-                printf("DCU_PREFETCHER:\t");
-                flags |= (1ULL<<37);
-                break;
+        case IP_PREFETCHER:
+            printf("IP_PREFETCHER:\t");
+            flags |= (1ULL<<39);
+            break;
 
-            case IP_PREFETCHER:
-                printf("IP_PREFETCHER:\t");
-                flags |= (1ULL<<39);
-                break;
-
-            default:
-                printf("ERROR: CpuFeature not supported!\n");
-                break;
-        }
-        PRINT_VALUE(RED,disabled);
-        printf("\n");
-
-        msr_write(cpu, MSR_IA32_MISC_ENABLE, flags);
-    }
-    else
-    {
-        printf("ERROR: Architecture does not support the manipulation of prefetchers\n");
+        default:
+            printf("ERROR: CpuFeature not supported!\n");
+            break;
     }
+    PRINT_VALUE(RED,disabled);
+    printf("\n");
+
+    HPMwrite(cpu, MSR_DEV, MSR_IA32_MISC_ENABLE, flags);
 }
 
