@@ -6,13 +6,13 @@
  *      Description: Hashtable implementation based on SGLIB.
  *                   Used for Marker API result handling.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,20 @@ void hashTable_init()
     }
 }
 
+void hashTable_initThread(int coreID)
+{
+    ThreadList* resPtr = threadList[coreID];
+    /* check if thread was already initialized */
+    if (resPtr == NULL)
+    {
+        resPtr = (ThreadList*) malloc(sizeof(ThreadList));
+        /* initialize structure */
+        resPtr->tid =  pthread_self();
+        resPtr->coreId  = coreID;
+        resPtr->hashTable = g_hash_table_new(g_str_hash, g_str_equal);
+        threadList[coreID] = resPtr;
+    }
+}
 
 int hashTable_get(bstring label, LikwidThreadResults** resEntry)
 {
@@ -86,7 +100,7 @@ int hashTable_get(bstring label, LikwidThreadResults** resEntry)
         (*resEntry)->label = bstrcpy (label);
         (*resEntry)->time = 0.0;
         (*resEntry)->count = 0;
-        for (int i=0; i< NUM_PMC; i++) 
+        for (int i=0; i< NUM_PMC; i++)
         {
             (*resEntry)->PMcounters[i] = 0.0;
             (*resEntry)->StartPMcounters[i] = 0.0;
@@ -109,7 +123,6 @@ void hashTable_finalize(int* numThreads, int* numRegions, LikwidResults** result
     GHashTable* regionLookup;
 
     regionLookup = g_hash_table_new(g_str_hash, g_str_equal);
-
     /* determine number of active threads */
     for (int i=0; i<MAX_NUM_THREADS; i++)
     {
@@ -128,22 +141,50 @@ void hashTable_finalize(int* numThreads, int* numRegions, LikwidResults** result
 
     /* allocate data structures */
     (*results) = (LikwidResults*) malloc(numberOfRegions * sizeof(LikwidResults));
-
-    for ( uint32_t i=0; i < numberOfRegions; i++ )
+    if (!(*results))
     {
-        (*results)[i].time = (double*) malloc(numberOfThreads * sizeof(double));
-        (*results)[i].count = (uint32_t*) malloc(numberOfThreads * sizeof(uint32_t));
-        (*results)[i].counters = (double**) malloc(numberOfThreads * sizeof(double*));
-
-        for ( uint32_t j=0; j < numberOfThreads; j++ )
+        fprintf(stderr, "Failed to allocate %lu bytes for the results\n", numberOfRegions * sizeof(LikwidResults));
+    }
+    else
+    {
+        for ( uint32_t i=0; i < numberOfRegions; i++ )
         {
-            (*results)[i].time[j] = 0.0;
-            (*results)[i].count[j] = 0;
-            (*results)[i].counters[j] = (double*) malloc(NUM_PMC * sizeof(double));
-
-            for ( uint32_t k=0; k < NUM_PMC; k++ )
+            (*results)[i].time = (double*) malloc(numberOfThreads * sizeof(double));
+            if (!(*results)[i].time)
             {
-                (*results)[i].counters[j][k] = 0.0;
+                fprintf(stderr, "Failed to allocate %lu bytes for the time storage\n", numberOfThreads * sizeof(double));
+                break;
+            }
+            (*results)[i].count = (uint32_t*) malloc(numberOfThreads * sizeof(uint32_t));
+            if (!(*results)[i].count)
+            {
+                fprintf(stderr, "Failed to allocate %lu bytes for the count storage\n", numberOfThreads * sizeof(uint32_t));
+                break;
+            }
+            (*results)[i].counters = (double**) malloc(numberOfThreads * sizeof(double*));
+            if (!(*results)[i].counters)
+            {
+                fprintf(stderr, "Failed to allocate %lu bytes for the counter result storage\n", numberOfThreads * sizeof(double*));
+                break;
+            }
+
+            for ( uint32_t j=0; j < numberOfThreads; j++ )
+            {
+                (*results)[i].time[j] = 0.0;
+                (*results)[i].count[j] = 0;
+                (*results)[i].counters[j] = (double*) malloc(NUM_PMC * sizeof(double));
+                if (!(*results)[i].counters)
+                {
+                    fprintf(stderr, "Failed to allocate %lu bytes for the counter result storage for thread %d\n", NUM_PMC * sizeof(double), j);
+                    break;
+                }
+                else
+                {
+                    for ( uint32_t k=0; k < NUM_PMC; k++ )
+                    {
+                        (*results)[i].counters[j][k] = 0.0;
+                    }
+                }
             }
         }
     }
@@ -174,6 +215,7 @@ void hashTable_finalize(int* numThreads, int* numRegions, LikwidResults** result
                 if ( regionId == NULL )
                 {
                     (*results)[currentRegion].tag = bstrcpy (threadResult->label);
+                    (*results)[currentRegion].groupID = threadResult->groupID;
                     regionIds[currentRegion] = currentRegion;
                     regionId = regionIds + currentRegion;
                     g_hash_table_insert(regionLookup, g_strdup(key), (regionIds+currentRegion));
