@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for accessClient module.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,12 +33,6 @@
 
 #include <stdint.h>
 
-/* This naming with AccessType and AccessMode is admittedly a bit confusing */
-typedef enum {
-    DAEMON_AM_DIRECT = 0,
-    DAEMON_AM_ACCESS_D
-} AccessMode;
-
 typedef enum {
     DAEMON_READ = 0,
     DAEMON_WRITE,
@@ -70,9 +64,7 @@ typedef enum {
     ERR_OPENFAIL,     /* failure to open msr files */
     ERR_RWFAIL,       /* failure to read/write msr */
     ERR_DAEMONBUSY,   /* daemon already has another client */
-    ERR_LOCKED,       /* access to HPM is locked */
-    ERR_UNSUPPORTED,   /* unsupported processor */
-    ERR_NODEV	/* No such device */
+    ERR_NODEV         /* No such device */
 } AccessErrorType;
 
 typedef struct {
@@ -84,4 +76,6 @@ typedef struct {
     AccessErrorType errorcode; /* Only in replies - 0 if no error. */
 } AccessDataRecord;
 
+extern int accessClient_mode;
+
 #endif /*ACCESSCLIENT_TYPES_H*/
