@@ -5,13 +5,14 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -32,17 +33,18 @@
 #define AFFINITY_H
 
 #include <types.h>
+#include <likwid.h>
+
+int socket_lock[MAX_NUM_NODES];
+int tile_lock[MAX_NUM_THREADS];
+extern AffinityDomains affinityDomains;
 
 extern int affinity_core2node_lookup[MAX_NUM_THREADS];
 
-extern void affinity_init();
-extern void affinity_finalize();
-extern int  affinity_processGetProcessorId();
-extern int  affinity_threadGetProcessorId();
-extern void  affinity_pinProcess(int processorId);
-extern void  affinity_pinThread(int processorId);
+extern int affinity_processGetProcessorId();
+extern int affinity_threadGetProcessorId();
 extern const AffinityDomain* affinity_getDomain(bstring domain);
-extern void affinity_printDomains(FILE* OUTSTREAM);
+
 
 #endif /*AFFINITY_H*/
 
