@@ -5,13 +5,14 @@
  *
  *      Description:  Central error handling macros
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,60 +32,81 @@
 #ifndef ERROR_H
 #define ERROR_H
 
-#include <errno.h>
-#include <string.h>
-#include <msr.h>
+
+#include <likwid.h>
+
+
 
 #define str(x) #x
 
-#define FINALIZE  msr_finalize()
 
 #define ERRNO_PRINT fprintf(stderr, "ERROR - [%s:%d] %s\n", __FILE__, __LINE__, strerror(errno))
 
 #define ERROR  \
     ERRNO_PRINT; \
-    FINALIZE; \
     exit(EXIT_FAILURE)
 
 #define ERROR_PLAIN_PRINT(msg) \
-   fprintf(stderr,  "ERROR - [%s:%d] " str(msg) "\n", __FILE__, __LINE__);  \
-   FINALIZE; \
-   exit(EXIT_FAILURE)
+   fprintf(stderr,  "ERROR - [%s:%s:%d] " str(msg) "\n", __FILE__, __func__,__LINE__);
 
 
 #define ERROR_PRINT(fmt, ...) \
-   fprintf(stderr,  "ERROR - [%s:%d] " str(fmt) "\n", __FILE__, __LINE__, __VA_ARGS__);  \
-   FINALIZE; \
-   exit(EXIT_FAILURE)
+   fprintf(stderr,  "ERROR - [%s:%s:%d] %s.\n" str(fmt) "\n", __FILE__,  __func__,__LINE__, strerror(errno), __VA_ARGS__);
 
 #define CHECK_ERROR(func, msg)  \
     if ((func) < 0) { \
         fprintf(stderr, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno));  \
     }
 
+#define CHECK_AND_RETURN_ERROR(func, msg)  \
+    if ((func) < 0) { \
+        fprintf(stderr, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno));  \
+        return errno; \
+    }
+
 #define EXIT_IF_ERROR(func, msg)  \
     if ((func) < 0) {  \
         fprintf(stderr,"ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
-        FINALIZE; \
         exit(EXIT_FAILURE); \
     }
 
-#ifndef DEBUGLEV
-#define DEBUGLEV 0
-#endif
+
 
 #define VERBOSEPRINTREG(cpuid,reg,flags,msg) \
-    if (perfmon_verbose) {  \
+    if (perfmon_verbosity >= DEBUGLEV_DETAIL) \
+    { \
         printf("DEBUG - [%s:%d] "  str(msg) " [%d] Register 0x%llX , Flags: 0x%llX \n",  \
-                __FILE__, __LINE__,  (cpuid), LLU_CAST (reg), LLU_CAST (flags)); \
+                __func__, __LINE__,  (cpuid), LLU_CAST (reg), LLU_CAST (flags)); \
         fflush(stdout);  \
-    } 
+    }
+    
+#define VERBOSEPRINTPCIREG(cpuid,dev,reg,flags,msg) \
+    if (perfmon_verbosity >= DEBUGLEV_DETAIL) \
+    { \
+        printf("DEBUG - [%s:%d] "  str(msg) " [%d] Device %d Register 0x%llX , Flags: 0x%llX \n",  \
+                __func__, __LINE__,  (cpuid), dev, LLU_CAST (reg), LLU_CAST (flags)); \
+        fflush(stdout);  \
+    }
 
 
 #define DEBUG_PRINT(lev, fmt, ...) \
-    if (DEBUGLEV > lev) { \
-        printf(fmt, __VA_ARGS__); \
+    if ((lev >= 0) && (lev <= perfmon_verbosity)) { \
+        fprintf(stdout, "DEBUG - [%s:%d] " str(fmt) "\n", __func__, __LINE__,__VA_ARGS__); \
         fflush(stdout); \
     }
 
+#define DEBUG_PLAIN_PRINT(lev, msg) \
+    if ((lev >= 0) && (lev <= perfmon_verbosity)) { \
+        fprintf(stdout, "DEBUG - [%s:%d] " str(msg) "\n",__func__, __LINE__);  \
+        fflush(stdout); \
+    }
+
+
+#define CHECK_MSR_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, MSR write operation failed);
+#define CHECK_MSR_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, MSR read operation failed);
+#define CHECK_PCI_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, PCI write operation failed);
+#define CHECK_PCI_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, PCI read operation failed);
+#define CHECK_POWER_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, Power register read operation failed);
+#define CHECK_TEMP_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, Temperature register read operation failed);
+
 #endif /*ERROR_H*/
