@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for libperfctr module.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,6 +37,7 @@ typedef struct LikwidThreadResults{
     bstring  label;
     double time;
     TimerData startTime;
+    int groupID;
     uint32_t count;
     double StartPMcounters[NUM_PMC];
     double PMcounters[NUM_PMC];
@@ -44,6 +45,7 @@ typedef struct LikwidThreadResults{
 
 typedef struct {
     bstring  tag;
+    int groupID;
     double*  time;
     uint32_t*  count;
     double** counters;
