@@ -3,15 +3,16 @@
  *
  *      Filename:  memsweep.h
  *
- *      Description:  Header File memsweep Module. 
+ *      Description:  Header File memsweep module for internal use. External functions are
+ *                    defined in likwid.h
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,9 +35,7 @@
 #include <types.h>
 
 extern void memsweep_setMemoryFraction(uint64_t fraction);
-extern void memsweep_node(FILE* OUTSTREAM);
-extern void memsweep_domain(FILE* OUTSTREAM, int domainId);
-extern void memsweep_threadGroup(FILE* OUTSTREAM, int* processorList, int numberOfProcessors);
+extern void memsweep_node(void);
 
 #endif /* MEMSWEEP_H */
 
