@@ -3,15 +3,16 @@
  *
  *      Filename:  numa.h
  *
- *      Description:  Header File numa Module. 
+ *      Description:  Header File NUMA module for internal use. External functions are
+ *                    defined in likwid.h
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -28,18 +29,30 @@
  * =======================================================================================
  */
 
-#ifndef NUMA_H
-#define NUMA_H
+#ifndef LIKWID_NUMA
+#define LIKWID_NUMA
+
+#include <stdlib.h>
+#include <stdio.h>
 
 #include <types.h>
+#include <likwid.h>
+#include <numa_hwloc.h>
+#include <numa_proc.h>
 
-/** Structure holding numa information
- *
- */
-extern NumaTopology numa_info;
 
-extern int numa_init (void);
-extern void numa_setInterleaved(int* processorList, int numberOfProcessors);
-extern void numa_membind(void* ptr, size_t size, int domainId);
 
-#endif /*NUMA_H*/
+
+extern int str2int(const char* str);
+
+struct numa_functions {
+    int (*numa_init) (void);
+    void (*numa_setInterleaved) (int*, int);
+    void (*numa_membind) (void*, size_t, int);
+};
+
+
+
+
+
+#endif
