@@ -3,15 +3,15 @@
  *
  *      Filename:  pci.h
  *
- *      Description:  Header File pci Module. 
+ *      Description:  Header File pci Module.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,11 +39,12 @@
  * Listing for Uncore devices DEVICE.FUNCTION
  */
 
-extern void pci_init();
+extern int pci_init(int socket_fd);
 extern void pci_finalize();
-extern uint32_t pci_read(int cpu, PciDeviceIndex index, uint32_t reg);
-extern void pci_write(int cpu, PciDeviceIndex index, uint32_t reg, uint32_t data);
-extern uint32_t pci_tread(int socket_fd, int cpu, PciDeviceIndex index, uint32_t reg);
-extern void pci_twrite(int socket_fd, int cpu, PciDeviceIndex index, uint32_t reg, uint32_t data);
+extern int pci_checkDevice(PciDeviceIndex index, int cpu);
+extern int pci_read(int cpu, PciDeviceIndex index, uint32_t reg, uint32_t *data);
+extern int pci_write(int cpu, PciDeviceIndex index, uint32_t reg, uint32_t data);
+extern int pci_tread(int socket_fd, int cpu, PciDeviceIndex index, uint32_t reg, uint32_t *data);
+extern int pci_twrite(int socket_fd, int cpu, PciDeviceIndex index, uint32_t reg, uint32_t data);
 
 #endif /* PCI_H */
