@@ -5,13 +5,13 @@
  *
  *      Description:  Header file of perfmon module for Atom
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -29,8 +29,7 @@
  */
 
 #include <perfmon_atom_events.h>
-#include <perfmon_atom_groups.h>
+#include <error.h>
 
-static int perfmon_numGroupsAtom = NUM_GROUPS_ATOM;
 static int perfmon_numArchEventsAtom = NUM_ARCH_EVENTS_ATOM;
 
