@@ -3,17 +3,17 @@
  *
  *      Filename:  perfmon_k8.h
  *
- *      Description:  Header File of perfmon module for K8 support.
- *                    Configures and reads out performance counters
- *                    on x86 based architectures. Supports multi threading.
+ *      Description:  Header File of perfmon module for AMD K8 support.
+ *                    The setup routines and registers are similar to AMD K10
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,10 +31,9 @@
  */
 
 #include <perfmon_k8_events.h>
-#include <perfmon_k8_groups.h>
+#include <error.h>
 
 
-static int perfmon_numGroupsK8 = NUM_GROUPS_K8;
 static int perfmon_numArchEventsK8 = NUM_ARCH_EVENTS_K8;
 
 
