@@ -3,15 +3,16 @@
  *
  *      Filename:  perfmon_pm_counters.h
  *
- *      Description: Counter Header File of perfmon module.
+ *      Description: Counter Header File of perfmon module for Intel Pentium M.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,8 +32,13 @@
 #define NUM_COUNTERS_PM 2
 #define NUM_COUNTERS_CORE_PM 2
 
-static PerfmonCounterMap pm_counter_map[NUM_COUNTERS_PM] = {
-    {"PMC0",PMC0, PMC, MSR_PERFEVTSEL0, MSR_PMC0, 0, 0},
-    {"PMC1",PMC1, PMC, MSR_PERFEVTSEL1, MSR_PMC1, 0, 0}
+#define PM_VALID_OPTIONS_PMC EVENT_OPTION_EDGE_MASK|EVENT_OPTION_COUNT_KERNEL_MASK|EVENT_OPTION_THRESHOLD_MASK|EVENT_OPTION_INVERT_MASK
+
+static RegisterMap pm_counter_map[NUM_COUNTERS_PM] = {
+    {"PMC0", PMC0, PMC, MSR_PERFEVTSEL0, MSR_PMC0, 0, 0, PM_VALID_OPTIONS_PMC},
+    {"PMC1", PMC1, PMC, MSR_PERFEVTSEL1, MSR_PMC1, 0, 0, PM_VALID_OPTIONS_PMC}
 };
 
+static BoxMap pm_box_map[NUM_UNITS] = {
+    [PMC] = {0, 0, 0, 0, 0, 0, 48}
+};
