@@ -3,15 +3,16 @@
  *
  *      Filename:  perfmon_westmere.h
  *
- *      Description:  Header File of perfmon module for Westmere.
+ *      Description:  Header File of perfmon module for Intel Westmere.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -29,8 +30,6 @@
  */
 
 #include <perfmon_westmere_events.h>
-#include <perfmon_westmere_groups.h>
 
-static int perfmon_numGroupsWestmere = NUM_GROUPS_WESTMERE;
 static int perfmon_numArchEventsWestmere = NUM_ARCH_EVENTS_WESTMERE;
 
