@@ -5,13 +5,14 @@
  *
  *      Description:  Types file for power module.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -32,36 +33,10 @@
 #define POWER_TYPES_H
 
 #include <stdint.h>
+#include <likwid.h>
 
-typedef enum {
-    PKG = 0,
-    PP0,
-    PP1,
-    DRAM
-} PowerType;
 
-typedef struct {
-    int numSteps;
-    double* steps;
-} TurboBoost;
-
-typedef struct {
-    double baseFrequency;
-    double minFrequency;
-    TurboBoost turbo;
-    double powerUnit;
-    double energyUnit;
-    double timeUnit;
-    double tdp;
-    double minPower;
-    double maxPower;
-    double maxTimeWindow;
-} PowerInfo;
-
-typedef struct {
-    uint32_t before;
-    uint32_t after;
-} PowerData;
+extern uint32_t power_regs[NUM_POWER_DOMAINS];
 
 
 #endif /*POWER_TYPES_H*/
