@@ -5,13 +5,14 @@
  *
  *      Description:  Register Defines for the perfmon module
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
+ *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -47,6 +48,10 @@
 #define MSR_PERFEVTSEL1           0x187
 #define MSR_PERFEVTSEL2           0x188
 #define MSR_PERFEVTSEL3           0x189
+#define MSR_PERFEVTSEL4           0x190
+#define MSR_PERFEVTSEL5           0x191
+#define MSR_PERFEVTSEL6           0x192
+#define MSR_PERFEVTSEL7           0x193
 #define MSR_PMC0                  0x0C1
 #define MSR_PMC1                  0x0C2
 #define MSR_PMC2                  0x0C3
@@ -60,6 +65,7 @@
 #define MSR_PERF_GLOBAL_STATUS    0x38E
 #define MSR_PERF_GLOBAL_OVF_CTRL  0x390
 #define MSR_PEBS_ENABLE           0x3F1
+#define MSR_PEBS_LD_LAT           0x3F6
 /* Perfmon V3 */
 #define MSR_OFFCORE_RESP0              0x1A6
 #define MSR_OFFCORE_RESP1              0x1A7
@@ -85,20 +91,21 @@
 #define MSR_UNCORE_PMC5                0x3B5
 #define MSR_UNCORE_PMC6                0x3B6
 #define MSR_UNCORE_PMC7                0x3B7
-/*
- * Perfmon V4 (starting with Haswell, according to
+/* 
+ * Perfmon V4 (starting with Haswell, according to 
  * Intel software developers guide also for SandyBridge,
- * IvyBridge not mentioned in this section)
+ * IvyBridge not mentioned in this section) 
  */
 #define MSR_UNC_PERF_GLOBAL_CTRL       MSR_UNCORE_PERF_GLOBAL_CTRL
 #define MSR_UNC_PERF_GLOBAL_STATUS     MSR_UNCORE_PERF_GLOBAL_STATUS
+#define MSR_UNC_PERF_GLOBAL_OVF_CTRL   MSR_UNCORE_PERF_GLOBAL_OVF_CTRL
 #define MSR_UNC_PERF_FIXED_CTRL        MSR_UNCORE_FIXED_CTR0
 #define MSR_UNC_PERF_FIXED_CTR         MSR_UNCORE_FIXED_CTR_CTRL
 #define MSR_UNC_ARB_PERFEVTSEL0        MSR_UNCORE_PMC2
 #define MSR_UNC_ARB_PERFEVTSEL1        MSR_UNCORE_PMC3
 #define MSR_UNC_ARB_CTR0               MSR_UNCORE_PMC0
 #define MSR_UNC_ARB_CTR1               MSR_UNCORE_PMC1
-#define MSR_UNC_CBO_CONFIG             0x396
+#define MSR_UNC_CBO_CONFIG             MSR_UNCORE_ADDR_OPCODE_MATCH
 #define MSR_UNC_CBO_0_PERFEVTSEL0      0x700
 #define MSR_UNC_CBO_0_PERFEVTSEL1      0x701
 #define MSR_UNC_CBO_0_CTR0             0x706
@@ -324,7 +331,10 @@
 #define MSR_UNC_PCU_PMON_CTL2           0xC32
 #define MSR_UNC_PCU_PMON_CTL3           0xC33
 #define MSR_UNC_PCU_PMON_BOX_FILTER     0xC34
-#define MSR_UNC_PCU_PMON_BOX_CTL        0xD24
+#define MSR_UNC_PCU_PMON_BOX_CTL        0xC24
+#define MSR_UNC_PCU_PMON_BOX_STATUS     0xC35
+#define MSR_UNC_PCU_PMON_FIXED_CTR0     0x3FC
+#define MSR_UNC_PCU_PMON_FIXED_CTR1     0x3FD
 
 /* UBox Performance Monitoring */
 
@@ -342,6 +352,7 @@
 /* HA Box Performance Monitoring */
 
 #define PCI_UNC_HA_PMON_BOX_CTL         0xF4
+#define PCI_UNC_HA_PMON_BOX_STATUS      0xF8
 #define PCI_UNC_HA_PMON_CTL_0           0xD8
 #define PCI_UNC_HA_PMON_CTL_1           0xDC
 #define PCI_UNC_HA_PMON_CTL_2           0xE0
@@ -378,9 +389,22 @@
 #define PCI_UNC_MC_PMON_CTR_2_B         0xB0
 #define PCI_UNC_MC_PMON_CTR_3_B         0xB8
 
+/* IRP Performance Monitoring */
+#define PCI_UNC_IRP_PMON_BOX_STATUS     0xF8
+#define PCI_UNC_IRP_PMON_BOX_CTL        0xF4
+#define PCI_UNC_IRP0_PMON_CTL_0         0xD8
+#define PCI_UNC_IRP0_PMON_CTL_1         0xDC
+#define PCI_UNC_IRP0_PMON_CTR_0         0xA0
+#define PCI_UNC_IRP0_PMON_CTR_1         0xB0
+#define PCI_UNC_IRP1_PMON_CTL_0         0xE0
+#define PCI_UNC_IRP1_PMON_CTL_1         0xE4
+#define PCI_UNC_IRP1_PMON_CTR_0         0xB8
+#define PCI_UNC_IRP1_PMON_CTR_1         0xC0
+
 /* QPI Box Performance Monitoring */
 
 #define PCI_UNC_QPI_PMON_BOX_CTL         0xF4
+#define PCI_UNC_QPI_PMON_BOX_STATUS      0xF8
 #define PCI_UNC_QPI_PMON_CTL_0           0xD8
 #define PCI_UNC_QPI_PMON_CTL_1           0xDC
 #define PCI_UNC_QPI_PMON_CTL_2           0xE0
@@ -402,6 +426,7 @@
 /* R2PCIE Box Performance Monitoring */
 
 #define PCI_UNC_R2PCIE_PMON_BOX_CTL         0xF4
+#define PCI_UNC_R2PCIE_PMON_BOX_STATUS      0xF8
 #define PCI_UNC_R2PCIE_PMON_CTL_0           0xD8
 #define PCI_UNC_R2PCIE_PMON_CTL_1           0xDC
 #define PCI_UNC_R2PCIE_PMON_CTL_2           0xE0
@@ -418,6 +443,7 @@
 /* R3QPI Box Performance Monitoring */
 
 #define PCI_UNC_R3QPI_PMON_BOX_CTL         0xF4
+#define PCI_UNC_R3QPI_PMON_BOX_STATUS      0xF8
 #define PCI_UNC_R3QPI_PMON_CTL_0           0xD8
 #define PCI_UNC_R3QPI_PMON_CTL_1           0xDC
 #define PCI_UNC_R3QPI_PMON_CTL_2           0xE0
@@ -428,6 +454,360 @@
 #define PCI_UNC_R3QPI_PMON_CTR_1_B         0xA8
 #define PCI_UNC_R3QPI_PMON_CTR_2_B         0xB0
 
+/* ########################################################## */
+/* Core v3 type uncore
+ * Naming following Intel Uncore Performance Monitoring Guide
+ * Ref. Nr. 331051-001
+ * */
+
+/* UBox Performance Monitoring */
+#define MSR_UNC_V3_U_PMON_CTR0             0x709
+#define MSR_UNC_V3_U_PMON_CTR1             0x70A
+#define MSR_UNC_V3_U_PMON_CTL0             0x705
+#define MSR_UNC_V3_U_PMON_CTL1             0x706
+#define MSR_UNC_V3_U_UCLK_FIXED_CTR        0x704
+#define MSR_UNC_V3_U_UCLK_FIXED_CTL        0x703
+#define MSR_UNC_V3_U_PMON_BOX_STATUS       0x708
+#define MSR_UNC_V3_U_PMON_GLOBAL_STATUS    0x701
+#define MSR_UNC_V3_U_PMON_GLOBAL_CTL       0x700
+#define MSR_UNC_V3_U_PMON_GLOBAL_CONFIG    0x702
+
+/* CBox Performance Monitoring */
+#define MSR_UNC_V3_C0_PMON_BOX_CTL         0xE00
+#define MSR_UNC_V3_C0_PMON_BOX_STATUS      0xE07
+#define MSR_UNC_V3_C0_PMON_BOX_FILTER0     0xE05
+#define MSR_UNC_V3_C0_PMON_BOX_FILTER1     0xE06
+#define MSR_UNC_V3_C0_PMON_CTL0            0xE01
+#define MSR_UNC_V3_C0_PMON_CTL1            0xE02
+#define MSR_UNC_V3_C0_PMON_CTL2            0xE03
+#define MSR_UNC_V3_C0_PMON_CTL3            0xE04
+#define MSR_UNC_V3_C0_PMON_CTR0            0xE08
+#define MSR_UNC_V3_C0_PMON_CTR1            0xE09
+#define MSR_UNC_V3_C0_PMON_CTR2            0xE0A
+#define MSR_UNC_V3_C0_PMON_CTR3            0xE0B
+
+#define MSR_UNC_V3_C1_PMON_BOX_CTL         0xE10
+#define MSR_UNC_V3_C1_PMON_BOX_STATUS      0xE17
+#define MSR_UNC_V3_C1_PMON_BOX_FILTER0     0xE15
+#define MSR_UNC_V3_C1_PMON_BOX_FILTER1     0xE16
+#define MSR_UNC_V3_C1_PMON_CTL0            0xE11
+#define MSR_UNC_V3_C1_PMON_CTL1            0xE12
+#define MSR_UNC_V3_C1_PMON_CTL2            0xE13
+#define MSR_UNC_V3_C1_PMON_CTL3            0xE14
+#define MSR_UNC_V3_C1_PMON_CTR0            0xE18
+#define MSR_UNC_V3_C1_PMON_CTR1            0xE19
+#define MSR_UNC_V3_C1_PMON_CTR2            0xE1A
+#define MSR_UNC_V3_C1_PMON_CTR3            0xE1B
+
+#define MSR_UNC_V3_C2_PMON_BOX_CTL         0xE20
+#define MSR_UNC_V3_C2_PMON_BOX_STATUS      0xE27
+#define MSR_UNC_V3_C2_PMON_BOX_FILTER0     0xE25
+#define MSR_UNC_V3_C2_PMON_BOX_FILTER1     0xE26
+#define MSR_UNC_V3_C2_PMON_CTL0            0xE21
+#define MSR_UNC_V3_C2_PMON_CTL1            0xE22
+#define MSR_UNC_V3_C2_PMON_CTL2            0xE23
+#define MSR_UNC_V3_C2_PMON_CTL3            0xE24
+#define MSR_UNC_V3_C2_PMON_CTR0            0xE28
+#define MSR_UNC_V3_C2_PMON_CTR1            0xE29
+#define MSR_UNC_V3_C2_PMON_CTR2            0xE2A
+#define MSR_UNC_V3_C2_PMON_CTR3            0xE2B
+
+#define MSR_UNC_V3_C3_PMON_BOX_CTL         0xE30
+#define MSR_UNC_V3_C3_PMON_BOX_STATUS      0xE37
+#define MSR_UNC_V3_C3_PMON_BOX_FILTER0     0xE35
+#define MSR_UNC_V3_C3_PMON_BOX_FILTER1     0xE36
+#define MSR_UNC_V3_C3_PMON_CTL0            0xE31
+#define MSR_UNC_V3_C3_PMON_CTL1            0xE32
+#define MSR_UNC_V3_C3_PMON_CTL2            0xE33
+#define MSR_UNC_V3_C3_PMON_CTL3            0xE34
+#define MSR_UNC_V3_C3_PMON_CTR0            0xE38
+#define MSR_UNC_V3_C3_PMON_CTR1            0xE39
+#define MSR_UNC_V3_C3_PMON_CTR2            0xE3A
+#define MSR_UNC_V3_C3_PMON_CTR3            0xE3B
+
+#define MSR_UNC_V3_C4_PMON_BOX_CTL         0xE40
+#define MSR_UNC_V3_C4_PMON_BOX_STATUS      0xE47
+#define MSR_UNC_V3_C4_PMON_BOX_FILTER0     0xE45
+#define MSR_UNC_V3_C4_PMON_BOX_FILTER1     0xE46
+#define MSR_UNC_V3_C4_PMON_CTL0            0xE41
+#define MSR_UNC_V3_C4_PMON_CTL1            0xE42
+#define MSR_UNC_V3_C4_PMON_CTL2            0xE43
+#define MSR_UNC_V3_C4_PMON_CTL3            0xE44
+#define MSR_UNC_V3_C4_PMON_CTR0            0xE48
+#define MSR_UNC_V3_C4_PMON_CTR1            0xE49
+#define MSR_UNC_V3_C4_PMON_CTR2            0xE4A
+#define MSR_UNC_V3_C4_PMON_CTR3            0xE4B
+
+#define MSR_UNC_V3_C5_PMON_BOX_CTL         0xE50
+#define MSR_UNC_V3_C5_PMON_BOX_STATUS      0xE57
+#define MSR_UNC_V3_C5_PMON_BOX_FILTER0     0xE55
+#define MSR_UNC_V3_C5_PMON_BOX_FILTER1     0xE56
+#define MSR_UNC_V3_C5_PMON_CTL0            0xE51
+#define MSR_UNC_V3_C5_PMON_CTL1            0xE52
+#define MSR_UNC_V3_C5_PMON_CTL2            0xE53
+#define MSR_UNC_V3_C5_PMON_CTL3            0xE54
+#define MSR_UNC_V3_C5_PMON_CTR0            0xE58
+#define MSR_UNC_V3_C5_PMON_CTR1            0xE59
+#define MSR_UNC_V3_C5_PMON_CTR2            0xE5A
+#define MSR_UNC_V3_C5_PMON_CTR3            0xE5B
+
+#define MSR_UNC_V3_C6_PMON_BOX_CTL         0xE60
+#define MSR_UNC_V3_C6_PMON_BOX_STATUS      0xE67
+#define MSR_UNC_V3_C6_PMON_BOX_FILTER0     0xE65
+#define MSR_UNC_V3_C6_PMON_BOX_FILTER1     0xE66
+#define MSR_UNC_V3_C6_PMON_CTL0            0xE61
+#define MSR_UNC_V3_C6_PMON_CTL1            0xE62
+#define MSR_UNC_V3_C6_PMON_CTL2            0xE63
+#define MSR_UNC_V3_C6_PMON_CTL3            0xE64
+#define MSR_UNC_V3_C6_PMON_CTR0            0xE68
+#define MSR_UNC_V3_C6_PMON_CTR1            0xE69
+#define MSR_UNC_V3_C6_PMON_CTR2            0xE6A
+#define MSR_UNC_V3_C6_PMON_CTR3            0xE6B
+
+#define MSR_UNC_V3_C7_PMON_BOX_CTL         0xE70
+#define MSR_UNC_V3_C7_PMON_BOX_STATUS      0xE77
+#define MSR_UNC_V3_C7_PMON_BOX_FILTER0     0xE75
+#define MSR_UNC_V3_C7_PMON_BOX_FILTER1     0xE76
+#define MSR_UNC_V3_C7_PMON_CTL0            0xE71
+#define MSR_UNC_V3_C7_PMON_CTL1            0xE72
+#define MSR_UNC_V3_C7_PMON_CTL2            0xE73
+#define MSR_UNC_V3_C7_PMON_CTL3            0xE74
+#define MSR_UNC_V3_C7_PMON_CTR0            0xE78
+#define MSR_UNC_V3_C7_PMON_CTR1            0xE79
+#define MSR_UNC_V3_C7_PMON_CTR2            0xE7A
+#define MSR_UNC_V3_C7_PMON_CTR3            0xE7B
+
+#define MSR_UNC_V3_C8_PMON_BOX_CTL         0xE80
+#define MSR_UNC_V3_C8_PMON_BOX_STATUS      0xE87
+#define MSR_UNC_V3_C8_PMON_BOX_FILTER0     0xE85
+#define MSR_UNC_V3_C8_PMON_BOX_FILTER1     0xE86
+#define MSR_UNC_V3_C8_PMON_CTL0            0xE81
+#define MSR_UNC_V3_C8_PMON_CTL1            0xE82
+#define MSR_UNC_V3_C8_PMON_CTL2            0xE83
+#define MSR_UNC_V3_C8_PMON_CTL3            0xE84
+#define MSR_UNC_V3_C8_PMON_CTR0            0xE88
+#define MSR_UNC_V3_C8_PMON_CTR1            0xE89
+#define MSR_UNC_V3_C8_PMON_CTR2            0xE8A
+#define MSR_UNC_V3_C8_PMON_CTR3            0xE8B
+
+#define MSR_UNC_V3_C9_PMON_BOX_CTL         0xE90
+#define MSR_UNC_V3_C9_PMON_BOX_STATUS      0xE97
+#define MSR_UNC_V3_C9_PMON_BOX_FILTER0     0xE95
+#define MSR_UNC_V3_C9_PMON_BOX_FILTER1     0xE96
+#define MSR_UNC_V3_C9_PMON_CTL0            0xE91
+#define MSR_UNC_V3_C9_PMON_CTL1            0xE92
+#define MSR_UNC_V3_C9_PMON_CTL2            0xE93
+#define MSR_UNC_V3_C9_PMON_CTL3            0xE94
+#define MSR_UNC_V3_C9_PMON_CTR0            0xE98
+#define MSR_UNC_V3_C9_PMON_CTR1            0xE99
+#define MSR_UNC_V3_C9_PMON_CTR2            0xE9A
+#define MSR_UNC_V3_C9_PMON_CTR3            0xE9B
+
+#define MSR_UNC_V3_C10_PMON_BOX_CTL        0xEA0
+#define MSR_UNC_V3_C10_PMON_BOX_STATUS     0xEA7
+#define MSR_UNC_V3_C10_PMON_BOX_FILTER0    0xEA5
+#define MSR_UNC_V3_C10_PMON_BOX_FILTER1    0xEA6
+#define MSR_UNC_V3_C10_PMON_CTL0           0xEA1
+#define MSR_UNC_V3_C10_PMON_CTL1           0xEA2
+#define MSR_UNC_V3_C10_PMON_CTL2           0xEA3
+#define MSR_UNC_V3_C10_PMON_CTL3           0xEA4
+#define MSR_UNC_V3_C10_PMON_CTR0           0xEA8
+#define MSR_UNC_V3_C10_PMON_CTR1           0xEA9
+#define MSR_UNC_V3_C10_PMON_CTR2           0xEAA
+#define MSR_UNC_V3_C10_PMON_CTR3           0xEAB
+
+#define MSR_UNC_V3_C11_PMON_BOX_CTL        0xEB0
+#define MSR_UNC_V3_C11_PMON_BOX_STATUS     0xEB7
+#define MSR_UNC_V3_C11_PMON_BOX_FILTER0    0xEB5
+#define MSR_UNC_V3_C11_PMON_BOX_FILTER1    0xEB6
+#define MSR_UNC_V3_C11_PMON_CTL0           0xEB1
+#define MSR_UNC_V3_C11_PMON_CTL1           0xEB2
+#define MSR_UNC_V3_C11_PMON_CTL2           0xEB3
+#define MSR_UNC_V3_C11_PMON_CTL3           0xEB4
+#define MSR_UNC_V3_C11_PMON_CTR0           0xEB8
+#define MSR_UNC_V3_C11_PMON_CTR1           0xEB9
+#define MSR_UNC_V3_C11_PMON_CTR2           0xEBA
+#define MSR_UNC_V3_C11_PMON_CTR3           0xEBB
+
+#define MSR_UNC_V3_C12_PMON_BOX_CTL        0xEC0
+#define MSR_UNC_V3_C12_PMON_BOX_STATUS     0xEC7
+#define MSR_UNC_V3_C12_PMON_BOX_FILTER0    0xEC5
+#define MSR_UNC_V3_C12_PMON_BOX_FILTER1    0xEC6
+#define MSR_UNC_V3_C12_PMON_CTL0           0xEC1
+#define MSR_UNC_V3_C12_PMON_CTL1           0xEC2
+#define MSR_UNC_V3_C12_PMON_CTL2           0xEC3
+#define MSR_UNC_V3_C12_PMON_CTL3           0xEC4
+#define MSR_UNC_V3_C12_PMON_CTR0           0xEC8
+#define MSR_UNC_V3_C12_PMON_CTR1           0xEC9
+#define MSR_UNC_V3_C12_PMON_CTR2           0xECA
+#define MSR_UNC_V3_C12_PMON_CTR3           0xECB
+
+#define MSR_UNC_V3_C13_PMON_BOX_CTL        0xED0
+#define MSR_UNC_V3_C13_PMON_BOX_STATUS     0xED7
+#define MSR_UNC_V3_C13_PMON_BOX_FILTER0    0xED5
+#define MSR_UNC_V3_C13_PMON_BOX_FILTER1    0xED6
+#define MSR_UNC_V3_C13_PMON_CTL0           0xED1
+#define MSR_UNC_V3_C13_PMON_CTL1           0xED2
+#define MSR_UNC_V3_C13_PMON_CTL2           0xED3
+#define MSR_UNC_V3_C13_PMON_CTL3           0xED4
+#define MSR_UNC_V3_C13_PMON_CTR0           0xED8
+#define MSR_UNC_V3_C13_PMON_CTR1           0xED9
+#define MSR_UNC_V3_C13_PMON_CTR2           0xEDA
+#define MSR_UNC_V3_C13_PMON_CTR3           0xEDB
+
+#define MSR_UNC_V3_C14_PMON_BOX_CTL        0xEE0
+#define MSR_UNC_V3_C14_PMON_BOX_STATUS     0xEE7
+#define MSR_UNC_V3_C14_PMON_BOX_FILTER0    0xEE5
+#define MSR_UNC_V3_C14_PMON_BOX_FILTER1    0xEE6
+#define MSR_UNC_V3_C14_PMON_CTL0           0xEE1
+#define MSR_UNC_V3_C14_PMON_CTL1           0xEE2
+#define MSR_UNC_V3_C14_PMON_CTL2           0xEE3
+#define MSR_UNC_V3_C14_PMON_CTL3           0xEE4
+#define MSR_UNC_V3_C14_PMON_CTR0           0xEE8
+#define MSR_UNC_V3_C14_PMON_CTR1           0xEE9
+#define MSR_UNC_V3_C14_PMON_CTR2           0xEEA
+#define MSR_UNC_V3_C14_PMON_CTR3           0xEEB
+
+#define MSR_UNC_V3_C15_PMON_BOX_CTL        0xEF0
+#define MSR_UNC_V3_C15_PMON_BOX_STATUS     0xEF7
+#define MSR_UNC_V3_C15_PMON_BOX_FILTER0    0xEF5
+#define MSR_UNC_V3_C15_PMON_BOX_FILTER1    0xEF6
+#define MSR_UNC_V3_C15_PMON_CTL0           0xEF1
+#define MSR_UNC_V3_C15_PMON_CTL1           0xEF2
+#define MSR_UNC_V3_C15_PMON_CTL2           0xEF3
+#define MSR_UNC_V3_C15_PMON_CTL3           0xEF4
+#define MSR_UNC_V3_C15_PMON_CTR0           0xEF8
+#define MSR_UNC_V3_C15_PMON_CTR1           0xEF9
+#define MSR_UNC_V3_C15_PMON_CTR2           0xEFA
+#define MSR_UNC_V3_C15_PMON_CTR3           0xEFB
+
+#define MSR_UNC_V3_C16_PMON_BOX_CTL        0xF00
+#define MSR_UNC_V3_C16_PMON_BOX_STATUS     0xF07
+#define MSR_UNC_V3_C16_PMON_BOX_FILTER0    0xF05
+#define MSR_UNC_V3_C16_PMON_BOX_FILTER1    0xF06
+#define MSR_UNC_V3_C16_PMON_CTL0           0xF01
+#define MSR_UNC_V3_C16_PMON_CTL1           0xF02
+#define MSR_UNC_V3_C16_PMON_CTL2           0xF03
+#define MSR_UNC_V3_C16_PMON_CTL3           0xF04
+#define MSR_UNC_V3_C16_PMON_CTR0           0xF08
+#define MSR_UNC_V3_C16_PMON_CTR1           0xF09
+#define MSR_UNC_V3_C16_PMON_CTR2           0xF0A
+#define MSR_UNC_V3_C16_PMON_CTR3           0xF0B
+
+#define MSR_UNC_V3_C17_PMON_BOX_CTL        0xF10
+#define MSR_UNC_V3_C17_PMON_BOX_STATUS     0xF17
+#define MSR_UNC_V3_C17_PMON_BOX_FILTER0    0xF15
+#define MSR_UNC_V3_C17_PMON_BOX_FILTER1    0xF16
+#define MSR_UNC_V3_C17_PMON_CTL0           0xF11
+#define MSR_UNC_V3_C17_PMON_CTL1           0xF12
+#define MSR_UNC_V3_C17_PMON_CTL2           0xF13
+#define MSR_UNC_V3_C17_PMON_CTL3           0xF14
+#define MSR_UNC_V3_C17_PMON_CTR0           0xF18
+#define MSR_UNC_V3_C17_PMON_CTR1           0xF19
+#define MSR_UNC_V3_C17_PMON_CTR2           0xF1A
+#define MSR_UNC_V3_C17_PMON_CTR3           0xF1B
+
+/* Sbox */
+#define MSR_UNC_V3_S0_PMON_BOX_CTL         0x720
+#define MSR_UNC_V3_S0_PMON_BOX_STATUS      0x725
+#define MSR_UNC_V3_S0_PMON_CTL_0           0x721
+#define MSR_UNC_V3_S0_PMON_CTL_1           0x722
+#define MSR_UNC_V3_S0_PMON_CTL_2           0x723
+#define MSR_UNC_V3_S0_PMON_CTL_3           0x724
+#define MSR_UNC_V3_S0_PMON_CTR_0           0x726
+#define MSR_UNC_V3_S0_PMON_CTR_1           0x727
+#define MSR_UNC_V3_S0_PMON_CTR_2           0x728
+#define MSR_UNC_V3_S0_PMON_CTR_3           0x729
+
+#define MSR_UNC_V3_S1_PMON_BOX_CTL         0x72A
+#define MSR_UNC_V3_S1_PMON_BOX_STATUS      0x72F
+#define MSR_UNC_V3_S1_PMON_CTL_0           0x72B
+#define MSR_UNC_V3_S1_PMON_CTL_1           0x72C
+#define MSR_UNC_V3_S1_PMON_CTL_2           0x72D
+#define MSR_UNC_V3_S1_PMON_CTL_3           0x72E
+#define MSR_UNC_V3_S1_PMON_CTR_0           0x730
+#define MSR_UNC_V3_S1_PMON_CTR_1           0x731
+#define MSR_UNC_V3_S1_PMON_CTR_2           0x732
+#define MSR_UNC_V3_S1_PMON_CTR_3           0x733
+
+#define MSR_UNC_V3_S2_PMON_BOX_CTL         0x734
+#define MSR_UNC_V3_S2_PMON_BOX_STATUS      0x739
+#define MSR_UNC_V3_S2_PMON_CTL_0           0x735
+#define MSR_UNC_V3_S2_PMON_CTL_1           0x736
+#define MSR_UNC_V3_S2_PMON_CTL_2           0x737
+#define MSR_UNC_V3_S2_PMON_CTL_3           0x738
+#define MSR_UNC_V3_S2_PMON_CTR_0           0x73A
+#define MSR_UNC_V3_S2_PMON_CTR_1           0x73B
+#define MSR_UNC_V3_S2_PMON_CTR_2           0x73C
+#define MSR_UNC_V3_S2_PMON_CTR_3           0x73D
+
+#define MSR_UNC_V3_S3_PMON_BOX_CTL         0x73E
+#define MSR_UNC_V3_S3_PMON_BOX_STATUS      0x743
+#define MSR_UNC_V3_S3_PMON_CTL_0           0x73F
+#define MSR_UNC_V3_S3_PMON_CTL_1           0x740
+#define MSR_UNC_V3_S3_PMON_CTL_2           0x741
+#define MSR_UNC_V3_S3_PMON_CTL_3           0x742
+#define MSR_UNC_V3_S3_PMON_CTR_0           0x744
+#define MSR_UNC_V3_S3_PMON_CTR_1           0x745
+#define MSR_UNC_V3_S3_PMON_CTR_2           0x746
+#define MSR_UNC_V3_S3_PMON_CTR_3           0x747
+
+/* V3 HA similar to V1/V2 */
+/* V3 iMC similar to V1/V2 */
+
+
+/* PCU (Power Control) Performance Monitoring */
+
+#define MSR_UNC_V3_PCU_PMON_CTR0           0x717
+#define MSR_UNC_V3_PCU_PMON_CTR1           0x718
+#define MSR_UNC_V3_PCU_PMON_CTR2           0x719
+#define MSR_UNC_V3_PCU_PMON_CTR3           0x71A
+#define MSR_UNC_V3_PCU_PMON_CTL0           0x711
+#define MSR_UNC_V3_PCU_PMON_CTL1           0x712
+#define MSR_UNC_V3_PCU_PMON_CTL2           0x713
+#define MSR_UNC_V3_PCU_PMON_CTL3           0x714
+#define MSR_UNC_V3_PCU_PMON_BOX_FILTER     0x715
+#define MSR_UNC_V3_PCU_PMON_BOX_CTL        0x710
+#define MSR_UNC_V3_PCU_PMON_BOX_STATUS     0x716
+#define MSR_UNC_V3_PCU_CC6_CTR             0x3FD
+#define MSR_UNC_V3_PCU_CC3_CTR             0x3FC
+#define MSR_UNC_V3_PCU_PC2_CTR             0x60D
+#define MSR_UNC_V3_PCU_PC3_CTR             0x3F8
+
+/* V3 QPI Box Performance Monitoring, mostly similar to V1/V2 */
+
+#define PCI_UNC_V3_QPI_PMON_BOX_CTL         0xF4
+#define PCI_UNC_V3_QPI_PMON_BOX_STATUS      0xF8
+#define PCI_UNC_V3_QPI_PMON_CTL_0           0xD8
+#define PCI_UNC_V3_QPI_PMON_CTL_1           0xDC
+#define PCI_UNC_V3_QPI_PMON_CTL_2           0xE0
+#define PCI_UNC_V3_QPI_PMON_CTL_3           0xE4
+#define PCI_UNC_V3_QPI_PMON_CTR_0_A         0xA4
+#define PCI_UNC_V3_QPI_PMON_CTR_1_A         0xAC
+#define PCI_UNC_V3_QPI_PMON_CTR_2_A         0xB4
+#define PCI_UNC_V3_QPI_PMON_CTR_3_A         0xBC
+#define PCI_UNC_V3_QPI_PMON_CTR_0_B         0xA0
+#define PCI_UNC_V3_QPI_PMON_CTR_1_B         0xA8
+#define PCI_UNC_V3_QPI_PMON_CTR_2_B         0xB0
+#define PCI_UNC_V3_QPI_PMON_CTR_3_B         0xB8
+#define PCI_UNC_V3_QPI_PMON_RX_MASK_0          0x238
+#define PCI_UNC_V3_QPI_PMON_RX_MASK_1          0x23C
+#define PCI_UNC_V3_QPI_PMON_RX_MATCH_0         0x228
+#define PCI_UNC_V3_QPI_PMON_RX_MATCH_1         0x22C
+#define PCI_UNC_V3_QPI_PMON_TX_MASK_0          0x210
+#define PCI_UNC_V3_QPI_PMON_TX_MASK_1          0x214
+#define PCI_UNC_V3_QPI_PMON_TX_MATCH_0         0x200
+#define PCI_UNC_V3_QPI_PMON_TX_MATCH_1         0x204
+#define PCI_UNC_V3_QPI_RATE_STATUS          0xD4
+#define PCI_UNC_V3_QPI_LINK_LLR             0xD0
+#define PCI_UNC_V3_QPI_LINK_IDLE            0xC8
+
+
+/* V3 R2PCIE Box Performance Monitoring similar to V1/V2 */
+
+/* V3 R3QPI Box Performance Monitoring similar to V1/V2 */
+
+/* ########################################################## */
 
 /* EX type uncore */
 /* U box - System Config Controller */
@@ -774,6 +1154,7 @@
 /* Match/Mask MSRs */
 #define MSR_B0_PMON_MATCH               0xE45
 #define MSR_B0_PMON_MASK                0xE46
+#define MSR_S0_PMON_MM_CFG              0xE49
 #define MSR_S0_PMON_MATCH               0xE49
 #define MSR_S0_PMON_MASK                0xE4A
 #define MSR_B1_PMON_MATCH               0xE4D
@@ -781,6 +1162,7 @@
 #define MSR_M0_PMON_MM_CONFIG           0xE54
 #define MSR_M0_PMON_ADDR_MATCH          0xE55
 #define MSR_M0_PMON_ADDR_MASK           0xE56
+#define MSR_S1_PMON_MM_CFG              0xE58
 #define MSR_S1_PMON_MATCH               0xE59
 #define MSR_S1_PMON_MASK                0xE5A
 #define MSR_M1_PMON_MM_CONFIG           0xE5C
@@ -803,6 +1185,8 @@
 #define MSR_DRAM_ENERGY_STATUS          0x619
 #define MSR_DRAM_PERF_STATUS            0x61B
 #define MSR_DRAM_POWER_INFO             0x61C
+/* Intel Silvermont's RAPL registers */
+#define MSR_PKG_POWER_INFO_SILVERMONT   0x66E
 
 /* TM/TM2 interface */
 #define IA32_THERM_STATUS               0x19C
@@ -814,9 +1198,9 @@
 #define MSR_PLATFORM_INFO               0x0CE
 #define MSR_TURBO_POWER_CURRENT_LIMIT   0x1AC
 #define MSR_TURBO_RATIO_LIMIT           0x1AD
+#define MSR_TURBO_RATIO_LIMIT1          0x1AE
+#define MSR_TURBO_RATIO_LIMIT2          0x1AF
 
-/* Intel Silvermont's RAPL registers */
-#define MSR_PKG_POWER_INFO_SILVERMONT   0x66E
 /*
  * AMD
  */
@@ -866,6 +1250,15 @@
 #define MSR_AMD16_PMC2                  0xC0010006
 #define MSR_AMD16_PMC3                  0xC0010007
 
+#define MSR_AMD16_L2_PERFEVTSEL0        0xC0010230
+#define MSR_AMD16_L2_PERFEVTSEL1        0xC0010232
+#define MSR_AMD16_L2_PERFEVTSEL2        0xC0010234
+#define MSR_AMD16_L2_PERFEVTSEL3        0xC0010236
+#define MSR_AMD16_L2_PMC0               0xC0010231
+#define MSR_AMD16_L2_PMC1               0xC0010233
+#define MSR_AMD16_L2_PMC2               0xC0010235
+#define MSR_AMD16_L2_PMC3               0xC0010237
+
 #define MSR_AMD16_NB_PERFEVTSEL0        0xC0010240
 #define MSR_AMD16_NB_PERFEVTSEL1        0xC0010242
 #define MSR_AMD16_NB_PERFEVTSEL2        0xC0010244
