@@ -6,13 +6,13 @@
  *      Description:  Header File Thermal Module.
  *                    Implements Intel TM/TM2 Interface.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -35,19 +35,43 @@
 #include <registers.h>
 #include <bitUtil.h>
 #include <msr.h>
+#include <error.h>
+#include <access.h>
 
-extern ThermalInfo thermal_info;
 
-extern void thermal_init(int cpuId);
-static inline uint32_t thermal_read(int cpuId);
 
-static uint32_t
-thermal_read(int cpuId)
+int
+thermal_read(int cpuId, uint32_t *data)
 {
-    uint32_t readout = extractBitField(msr_read(cpuId, IA32_THERM_STATUS),7,16);
-    return (readout == 0 ? 
-            thermal_info.activationT - thermal_info.offset :
-            (thermal_info.activationT-thermal_info.offset) - readout );
+    uint64_t result = 0;
+    uint32_t readout = 0;
+    if (HPMread(cpuId, MSR_DEV, IA32_THERM_STATUS, &result))
+    {
+        *data = 0;
+        return -EIO;
+    }
+    readout = extractBitField(result,7,16);
+    *data = (readout == 0 ?
+                thermal_info.activationT - thermal_info.offset :
+                (thermal_info.activationT - thermal_info.offset) - readout );
+    return 0;
+}
+
+int
+thermal_tread(int socket_fd, int cpuId, uint32_t *data)
+{
+    uint64_t result = 0;
+    uint32_t readout = 0;
+    if (HPMread(cpuId, MSR_DEV, IA32_THERM_STATUS, &result))
+    {
+        *data = 0;
+        return -EIO;
+    }
+    readout = extractBitField(result,7,16);
+    *data = (readout == 0 ?
+                thermal_info.activationT - thermal_info.offset :
+                (thermal_info.activationT - thermal_info.offset) - readout );
+    return 0;
 }
 
 #endif /*THERMAL_H*/
