@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for thermal module.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -33,7 +33,9 @@
 
 #include <stdint.h>
 
-
+/** \addtogroup ThermalMon
+ *  @{
+ */
 typedef struct {
     uint16_t highT;
     uint32_t resolution;
@@ -41,5 +43,11 @@ typedef struct {
     uint32_t offset;
 } ThermalInfo;
 
+/** \brief Pointer for exporting the ThermalInfo data structure */
+typedef ThermalInfo* ThermalInfo_t;
+/** @}*/
+
+extern ThermalInfo thermal_info;
+
 
 #endif /*THERMAL_TYPES_H*/
