@@ -6,13 +6,13 @@
  *      Description:  Header File tree Module. 
  *                    Implements a simple tree data structure.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,7 @@
 
 extern void tree_init(TreeNode** root, int id);
 extern void tree_print(TreeNode* nodePtr);
+extern void tree_destroy(TreeNode* nodePtr);
 extern void tree_insertNode(TreeNode* nodePtr, int id);
 extern int tree_nodeExists(TreeNode* nodePtr, int id);
 extern int tree_countChildren(TreeNode* nodePtr);
