@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for tree module.
  *
- *      Version:   3.0
- *      Released:  29.11.2012
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2012 Jan Treibig 
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,14 +31,24 @@
 #ifndef TREE_TYPES_H
 #define TREE_TYPES_H
 
-/* For arbitrary trees llink are the children and
- * rlink are the neighbours
- */
-typedef struct treeNode {
-    int id;
-    struct treeNode* llink;
-    struct treeNode* rlink;
-} TreeNode;
 
+/** \addtogroup CPUTopology
+*  @{
+*/
+/*! \brief Structure of a tree node
+
+This structure is used to form the tree of the system topology. The information
+describing each node is store in other places, therefore an ID is enough.
+\extends CpuTopology
+*/
+struct treeNode {
+    int id; /*!< \brief ID of the node */
+    struct treeNode* llink; /*!< \brief List of children of the current node */
+    struct treeNode* rlink; /*!< \brief List of neighbors of the current node */
+};
+
+/** \brief Shorter name for struct treeNode */
+typedef struct treeNode TreeNode;
+/** @}*/
 
 #endif /*TREE_TYPES_H*/
