@@ -5,13 +5,14 @@
  *
  *      Description: F90 interface for marker API
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:  Jan Treibig (jt), jan.treibig@gmail.com,
+ *               Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -48,6 +49,28 @@ void likwid_markerclose_(void)
     likwid_markerClose();
 }
 
+void likwid_markernextgroup_(void)
+{
+    likwid_markerNextGroup();
+}
+
+void likwid_markerregisterregion_(char* regionTag, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char) );
+    strncpy(tmp, regionTag, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+
+    likwid_markerRegisterRegion( tmp );
+    free(tmp);
+}
+
 void likwid_markerstartregion_(char* regionTag, int len)
 {
     char* tmp = (char*) malloc((len+1) * sizeof(char) );
@@ -82,3 +105,19 @@ void likwid_markerstopregion_(char* regionTag, int len)
     free(tmp);
 }
 
+void likwid_markergetregion_(char* regionTag, int* nr_events, double* events, double *time, int *count, int len)
+{
+    char* tmp = (char*) malloc((len+1) * sizeof(char));
+    strncpy(tmp, regionTag, len * sizeof(char) );
+
+    for (int i=(len-1); len > 0; len--)
+    {
+        if (tmp[i] != ' ') {
+            tmp[i+1] = 0;
+            break;
+        }
+    }
+    likwid_markerGetRegion( tmp, nr_events,  events, time, count);
+    free(tmp);
+}
+
