@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of sweeper module.
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,13 +37,12 @@
 #include <error.h>
 #include <types.h>
 #include <memsweep.h>
-#include <cpuid.h>
+#include <topology.h>
 #include <numa.h>
 #include <affinity.h>
 
 extern void _loadData(uint32_t size, void* ptr);
 
-
 /* #####   EXPORTED VARIABLES   ########################################### */
 
 
@@ -57,14 +56,14 @@ static uint64_t  memoryFraction = 80ULL;
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
-static void*
+static void* 
 allocateOnNode(size_t size, int domainId)
 {
-    char *ptr; 
+	char *ptr; 
 
-    ptr = mmap(0, size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, 0, 0);
+	ptr = mmap(0, size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS, 0, 0);  
 
-    if (ptr == (char *)-1)
+	if (ptr == (char *)-1)
     {
         ERROR;
     }
@@ -74,7 +73,7 @@ allocateOnNode(size_t size, int domainId)
     return ptr;
 }
 
-static void
+static void 
 initMemory(size_t size, char* ptr, int domainId)
 {
     affinity_pinProcess(numa_info.nodes[domainId].processors[0]);
@@ -101,20 +100,18 @@ findProcessor(uint32_t nodeId, uint32_t coreId)
 }
 
 /* evict all dirty cachelines from last level cache */
-static void cleanupCache(FILE* OUTSTREAM, char* ptr)
+static void cleanupCache(char* ptr)
 {
-#ifdef __x86_64
+#if defined(__x86_64__) || defined(__i386__)
     uint32_t cachesize = 2 * cpuid_topology.cacheLevels[cpuid_topology.numCacheLevels-1].size;
-    if (OUTSTREAM != NULL)
-    {
-        fprintf(OUTSTREAM, "Cleanup LLC using %u MB\n", cachesize / (1000000));
-    }
+    printf("Cleaning LLC with %g MB\n", (double)cachesize/(1024.0 * 1024.0));
     _loadData(cachesize,ptr);
 #else
-    ERROR_PLAIN_PRINT(Cleanup cache is currently only available on 64bit X86 systems.);
+    ERROR_PLAIN_PRINT(Cleanup cache is currently only available on X86 systems.);
 #endif
 }
 
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 void
@@ -125,35 +122,32 @@ memsweep_setMemoryFraction(uint64_t fraction)
 
 
 void
-memsweep_node(FILE* OUTSTREAM)
+memsweep_node(void)
 {
     for ( uint32_t i=0; i < numa_info.numberOfNodes; i++)
     {
-        memsweep_domain(OUTSTREAM, i);
+        memsweep_domain(i);
     }
 }
 
 
 void
-memsweep_domain(FILE* OUTSTREAM, int domainId)
+memsweep_domain(int domainId)
 {
     char* ptr = NULL;
     size_t size = numa_info.nodes[domainId].totalMemory * 1024ULL * memoryFraction / 100ULL;
-    if (OUTSTREAM != NULL)
-    {
-        fprintf(OUTSTREAM, "Sweeping domain %d: Using %g MB of %g MB\n",
-                domainId,
-                size / (1000.0 * 1000.0),
-                numa_info.nodes[domainId].totalMemory/ 1000.0);
-    }
+    printf("Sweeping domain %d: Using %g MB of %g MB\n",
+            domainId,
+            size / (1024.0 * 1024.0),
+            numa_info.nodes[domainId].totalMemory/ 1024.0);
     ptr = (char*) allocateOnNode(size, domainId);
     initMemory(size, ptr, domainId);
-    cleanupCache(OUTSTREAM, ptr);
+    cleanupCache(ptr);
     munmap(ptr, size);
 }
 
 void
-memsweep_threadGroup(FILE* OUTSTREAM, int* processorList, int numberOfProcessors)
+memsweep_threadGroup(int* processorList, int numberOfProcessors)
 {
     for (uint32_t i=0; i<numa_info.numberOfNodes; i++)
     {
@@ -161,10 +155,13 @@ memsweep_threadGroup(FILE* OUTSTREAM, int* processorList, int numberOfProcessors
         {
             if (findProcessor(i,processorList[j]))
             {
-                memsweep_domain(OUTSTREAM, i);
+                memsweep_domain(i);
                 break;
             }
         }
     }
 }
 
+
+
+
