@@ -5,13 +5,13 @@
  *
  *      Description:  Module implementing Intel TM/TM2 interface
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@
 
 #include <types.h>
 #include <thermal.h>
-#include <cpuid.h>
+#include <topology.h>
 
 /* #####   EXPORTED VARIABLES   ########################################### */
 
@@ -52,7 +52,10 @@ void thermal_init(int cpuId)
 
     if ( cpuid_hasFeature(TM2) )
     {
-        flags = msr_read(cpuId, IA32_THERM_STATUS);
+        if (HPMread(cpuId, MSR_DEV, IA32_THERM_STATUS, &flags))
+        {
+            return;
+        }
 
         if ( flags & 0x1 )
         {
@@ -66,7 +69,10 @@ void thermal_init(int cpuId)
         thermal_info.resolution =  extractBitField(flags,4,27);
 
         flags = 0ULL;
-        flags = msr_read(cpuId, MSR_TEMPERATURE_TARGET);
+        if (HPMread(cpuId, MSR_DEV, MSR_TEMPERATURE_TARGET, &flags))
+        {
+            return;
+        }
         thermal_info.activationT =  extractBitField(flags,8,16);
         thermal_info.offset = extractBitField(flags,6,24);
     }
