@@ -5,13 +5,13 @@
  *
  *      Description:  Module implementing a tree data structure
  *
- *      Version:   3.1.3
- *      Released:  4.11.2014
+ *      Version:   4.0
+ *      Released:  16.6.2015
  *
- *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
+ *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2014 Jan Treibig
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -48,32 +48,62 @@ tree_init(TreeNode** root, int id)
 void
 tree_print(TreeNode* nodePtr)
 {
-    int level = 0;
+  int level = 0;
+
+  if (nodePtr != NULL)
+  {
+
+    TreeNode* digger;
+    TreeNode* walker;
+
+    digger = nodePtr->llink;
+
+    while (digger != NULL)
+    {
+      printf("\n Level %d:\n", level++);
+      printf("%d ", digger->id);
+      walker = digger->rlink;
+
+      while (walker != NULL)
+      {
+        printf("%d ", walker->id);
+        walker = walker->rlink;
+      }
+
+      digger = digger->llink;
+    }
+
+    printf("\n ");
+  }
+}
+
+void
+tree_destroy(TreeNode* nodePtr)
+{
 
     if (nodePtr != NULL)
     {
 
         TreeNode* digger;
         TreeNode* walker;
+        TreeNode* tmp;
 
         digger = nodePtr->llink;
 
         while (digger != NULL)
         {
-            printf("\n Level %d:\n", level++);
-            printf("%d ", digger->id);
             walker = digger->rlink;
 
             while (walker != NULL)
             {
-            printf("%d ", walker->id);
-            walker = walker->rlink;
+                tmp = walker;
+                walker = walker->rlink;
+                free(tmp);
             }
-
+            tmp = digger;
             digger = digger->llink;
+            free(tmp);
         }
-
-        printf("\n ");
     }
 }
 
