@@ -24,10 +24,10 @@ testlist = "SET.txt"
 testfolder = "TESTS"
 resultfolder = "RESULTS"
 hostname = socket.gethostname()
-picture_base = "http://likwid.googlecode.com/svn/wiki/images"
+picture_base = ".."
 
-gnu_colors = ["red","blue","green"]#,"black","brown", "gray","violet", "cyan", "magenta","orange","#4B0082","#800000","turquoise","#006400","yellow"]
-gnu_marks = [5,13,9]#,2,3,4,6,7,8,9,10,11,12,14,15]
+gnu_colors = ["red","blue","green","black"]#,"brown", "gray","violet", "cyan", "magenta","orange","#4B0082","#800000","turquoise","#006400","yellow"]
+gnu_marks = [5,13,9,2]#,3,4,6,7,8,9,10,11,12,14,15]
 
 wiki = False
 papi = False
@@ -38,14 +38,27 @@ out_gnuplot = False
 out_grace = False
 scriptfilename = "create_plots.sh"
 out_script = False
+test_set = {}
+plain_set = {}
+corrected_set = {}
+marker_set = {}
+papi_set = {}
+
+if not os.path.exists(bench_plain) or not os.path.exists(bench_marker):
+    print "Please run make before using likwid-accuracy.py"
+    sys.exit(1)
+if not os.path.exists(perfctr):
+    print "Cannot find likwid-perfctr"
+    sys.exit(1)
+
 
 def usage():
     print "Execute and evaluate accuracy tests for LIKWID with likwid-bench and likwid-perfctr"
     print
     print "-h/--help:\tPrint this help text"
     print "-s/--sets:\tSpecifiy testgroups (comma separated). Can also be set in SET.txt"
-    print "--wiki:\t\tBesides testing write out results in Google code wiki syntax"
-    print "--only_wiki:\tDo not run benchmarks, read results from file and write out results in Google code wiki syntax"
+#    print "--wiki:\t\tBesides testing write out results in Google code wiki syntax"
+#    print "--only_wiki:\tDo not run benchmarks, read results from file and write out results in Google code wiki syntax"
     print "Picture options:"
     print "--pgf:\t\tCreate TeX document for each test with PGFPlot"
     print "--gnuplot:\tCreate GNUPlot script for each test"
@@ -87,8 +100,11 @@ def get_groups():
     for line in p.stdout.read().split("\n"):
         if line.startswith("-") or not line.strip(): continue
         if line.startswith("Available"): continue
-        name, description = line.split(":")
-        groups[name.strip()] = description.strip()
+        print(re.split("\s*", line.strip()))
+        linelist = re.split("\s+", line.strip())
+        name = linelist[0]
+        description = " ".join(linelist[1:])
+        groups[name] = description
     return groups
 
 def get_test_groups(groupdict):
@@ -130,7 +146,7 @@ def get_values_from_file(file, lineoffset, linecount):
             print line
     return results
 
-def write_pgf(group, test, plain_file, marker_file, papi_file=None, execute=False, script=None):
+def write_pgf(group, test, plain_file, correct_file, marker_file, papi_file=None, execute=False, script=None):
     filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".tex")
     fp = open(filename,'w')
     fp.write("\documentclass{article}\n")
@@ -141,10 +157,12 @@ def write_pgf(group, test, plain_file, marker_file, papi_file=None, execute=Fals
     fp.write("\\begin{axis}[xlabel={Run}, ylabel={MFlops/s / MBytes/s},title={%s\_%s},legend pos=south east,xtick=data,width=.75\\textwidth]\n" % (group.replace("_","\_"),test.replace("_","\_"),))
     fp.write("\\addplot+[red,mark=square*,mark options={draw=red, fill=red}] table {%s};\n" % (os.path.basename(plain_file),))
     fp.write("\\addlegendentry{plain};\n")
-    fp.write("\\addplot+[blue,mark=diamond*,mark options={draw=blue, fill=blue}] table {%s};\n" % (os.path.basename(marker_file),))
+    fp.write("\\addplot+[blue,mark=*,mark options={draw=blue, fill=blue}] table {%s};\n" % (os.path.basename(correct_file),))
+    fp.write("\\addlegendentry{corrected};\n")
+    fp.write("\\addplot+[green,mark=diamond*,mark options={draw=green, fill=green}] table {%s};\n" % (os.path.basename(marker_file),))
     fp.write("\\addlegendentry{marker};\n")
     if papi and papi_file:
-        fp.write("\\addplot+[green,mark=triangle*,mark options={draw=green, fill=green}] table {%s};\n" % (os.path.basename(papi_file),))
+        fp.write("\\addplot+[black,mark=triangle*,mark options={draw=black, fill=black}] table {%s};\n" % (os.path.basename(papi_file),))
         fp.write("\\addlegendentry{papi};\n")
     fp.write("\\end{axis}\n")
     fp.write("\\end{tikzpicture}\n")
@@ -162,7 +180,7 @@ def write_pgf(group, test, plain_file, marker_file, papi_file=None, execute=Fals
         script.write("pdflatex %s\n" % (os.path.basename(filename),))
     return filename
     
-def write_gnuplot(group, test, plain_file, marker_file, papi_file, execute=False, script=None):
+def write_gnuplot(group, test, plain_file, correct_file, marker_file, papi_file=None, execute=False, script=None):
     filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".plot")
     fp = open(filename,'w')
     for i,color in enumerate(gnu_colors):
@@ -170,12 +188,15 @@ def write_gnuplot(group, test, plain_file, marker_file, papi_file, execute=False
     fp.write("set terminal jpeg\n")
     fp.write("set title '%s_%s'\n" % (group, test,))
     fp.write("set output '%s'\n" % (os.path.basename(os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".jpg")),))
-    fp.write("set xlabel 'Run'\n")
+    fp.write("set xlabel 'Size - %d runs each'\n" % (test_set[group][test]["RUNS"],))
     fp.write("set ylabel 'MFlops/s / MBytes/s'\n")
-    #fp.write("set xtics 1\n")
-    plot_string = "plot '%s' using 1:2 title 'plain' with linespoints ls 1, \\\n '%s' using 1:2 title 'marker' with linespoints ls 2" % (os.path.basename(plain_file), os.path.basename(marker_file),)
+    #fp.write("set xtics 0,%d,%d\n" % (test_set[group][test]["RUNS"], test_set[group][test]["RUNS"]*len(test_set[group][test]["variants"]),))
+    fp.write("set xtics %d\n" % (test_set[group][test]["RUNS"]*len(test_set[group][test]["variants"]),))
+    for i,variant in enumerate(test_set[group][test]["variants"]):
+        fp.write("set xtics add (\"%s\" %f)\n" % (variant, (i*test_set[group][test]["RUNS"])+(0.5*test_set[group][test]["RUNS"]),))
+    plot_string = "plot '%s' using 1:2 title 'plain' with linespoints ls 1, \\\n '%s' using 1:2 title 'corrected' with linespoints ls 2, \\\n '%s' using 1:2 title 'marker' with linespoints ls 3" % (os.path.basename(plain_file), os.path.basename(correct_file), os.path.basename(marker_file),)
     if papi and papi_file:
-        plot_string += ", \\\n '%s' using 1:2 title 'papi' with linespoints ls 3\n" % (os.path.basename(papi_file),)
+        plot_string += ", \\\n '%s' using 1:2 title 'papi' with linespoints ls 4\n" % (os.path.basename(papi_file),)
     fp.write(plot_string+"\n")
     fp.close()
     if execute:
@@ -190,17 +211,18 @@ def write_gnuplot(group, test, plain_file, marker_file, papi_file, execute=False
         script.write("gnuplot %s\n" % (os.path.basename(filename),))
     return filename
 
-def write_grace(group, test, plain_file, marker_file, papi_file=None, execute=False, script=None):
+def write_grace(group, test, plain_file, correct_file, marker_file, papi_file=None, execute=False, script=None):
     filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".bat")
     agrname = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".agr")
     pngname = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+".png")
     if execute or script:
         plain_file = os.path.basename(plain_file)
         marker_file = os.path.basename(marker_file)
+        correct_file = os.path.basename(correct_file)
         if papi_file: papi_file = os.path.basename(papi_file)
         pngname = os.path.basename(pngname)
         agrname = os.path.basename(agrname)
-    cmd_options = "-autoscale xy -nxy %s -nxy %s "% (plain_file,marker_file,)
+    cmd_options = "-autoscale xy -nxy %s -nxy %s -nxy %s " % (plain_file, correct_file, marker_file,)
     if papi and papi_file:
         cmd_options += "-nxy %s " % (papi_file,)
     out_options = "-hdevice PNG -printfile %s " % (pngname,)
@@ -228,7 +250,7 @@ def write_grace(group, test, plain_file, marker_file, papi_file=None, execute=Fa
     fp.write("s0 line linestyle 1\n")
     fp.write("s0 line linewidth 2\n")
     fp.write("s0 line pattern 1\n")
-    fp.write("s1 legend \"marker\"\n")
+    fp.write("s1 legend \"corrected\"\n")
     fp.write("s1 symbol 3\n")
     fp.write("s1 symbol size 1\n")
     fp.write("s1 symbol color 4\n")
@@ -242,21 +264,35 @@ def write_grace(group, test, plain_file, marker_file, papi_file=None, execute=Fa
     fp.write("s1 line linestyle 1\n")
     fp.write("s1 line linewidth 2\n")
     fp.write("s1 line pattern 1\n")
+    fp.write("s2 legend \"marker\"\n")
+    fp.write("s2 symbol 4\n")
+    fp.write("s2 symbol size 1\n")
+    fp.write("s2 symbol color 3\n")
+    fp.write("s2 symbol pattern 1\n")
+    fp.write("s2 symbol fill color 3\n")
+    fp.write("s2 symbol fill pattern 1\n")
+    fp.write("s2 symbol linewidth 2\n")
+    fp.write("s2 symbol linestyle 1\n")
+    fp.write("s2 line type 1\n")
+    fp.write("s2 line color 3\n")
+    fp.write("s2 line linestyle 1\n")
+    fp.write("s2 line linewidth 2\n")
+    fp.write("s2 line pattern 1\n")
     if papi and papi_file:
-        fp.write("s2 legend \"papi\"\n")
-        fp.write("s2 symbol 4\n")
-        fp.write("s2 symbol size 1\n")
-        fp.write("s2 symbol color 3\n")
-        fp.write("s2 symbol pattern 1\n")
-        fp.write("s2 symbol fill color 3\n")
-        fp.write("s2 symbol fill pattern 1\n")
-        fp.write("s2 symbol linewidth 2\n")
-        fp.write("s2 symbol linestyle 1\n")
-        fp.write("s2 line type 1\n")
-        fp.write("s2 line color 3\n")
-        fp.write("s2 line linestyle 1\n")
-        fp.write("s2 line linewidth 2\n")
-        fp.write("s2 line pattern 1\n")
+        fp.write("s3 legend \"papi\"\n")
+        fp.write("s3 symbol 5\n")
+        fp.write("s3 symbol size 1\n")
+        fp.write("s3 symbol color \"black\"\n")
+        fp.write("s3 symbol pattern 1\n")
+        fp.write("s3 symbol fill color \"black\"\n")
+        fp.write("s3 symbol fill pattern 1\n")
+        fp.write("s3 symbol linewidth 2\n")
+        fp.write("s3 symbol linestyle 1\n")
+        fp.write("s3 line type 1\n")
+        fp.write("s3 line color \"black\"\n")
+        fp.write("s3 line linestyle 1\n")
+        fp.write("s3 line linewidth 2\n")
+        fp.write("s3 line pattern 1\n")
     fp.close()
     if execute:
         cmd = "cd %s && gracebat %s -param %s %s && cd -" % (os.path.dirname(filename), cmd_options, os.path.basename(filename),out_options,)
@@ -303,23 +339,27 @@ for o, a in opts:
     if o == "--scriptname":
         scriptfilename = a
 
-if not os.path.exists(testlist):
+if len(sets) == 0 and not os.path.exists(testlist):
     print "Cannot find file %s containing list of testgroups" % (testlist,)
     sys.exit(1)
 if not os.path.exists(testfolder):
     print "Cannot find folder %s containing the testgroups" % (testfolder,)
     sys.exit(1)
 
-test_set = {}
-plain_set = {}
-marker_set = {}
-papi_set = {}
-fp = open(testlist,'r')
-for line in fp.read().split("\n"):
+
+if len(sets) == 0:
+    fp = open(testlist,'r')
+    tmp = fp.read().split("\n")
+    for item in tmp:
+        if not item.strip() or item.startswith("#"): continue
+        sets.append(item)
+    fp.close()
+for line in sets:
     if not line.strip() or line.startswith("#"): continue
     if os.path.exists("%s/%s.txt" % (testfolder,line.strip(),)):
         test_set[line.strip()] = {}
         plain_set[line.strip()] = {}
+        corrected_set[line.strip()] = {}
         marker_set[line.strip()] = {}
         papi_set[line.strip()] = {}
         testfp = open("%s/%s.txt" % (testfolder,line.strip(),),'r')
@@ -335,11 +375,15 @@ for line in fp.read().split("\n"):
             if testline.startswith("TEST"):
                 test = testline.split(" ")[1]
                 test_set[line.strip()][test] = {}
+                test_set[line.strip()][test]["WA_FACTOR"] = 1.0
                 plain_set[line.strip()][test] = {}
+                corrected_set[line.strip()][test] = {}
                 marker_set[line.strip()][test] = {}
                 papi_set[line.strip()][test] = {}
             if testline.startswith("RUNS") and test:
                 test_set[line.strip()][test]["RUNS"] = int(testline.split(" ")[1])
+            if testline.startswith("WA_FACTOR") and test:
+                test_set[line.strip()][test]["WA_FACTOR"] = float(testline.split(" ")[1])
             if testline.startswith("VARIANT") and test:
                 linelist = re.split("\s+",testline);
                 variant = linelist[1]
@@ -348,10 +392,11 @@ for line in fp.read().split("\n"):
                 test_set[line.strip()][test][variant] = linelist[2]
                 test_set[line.strip()][test]["variants"].append(linelist[1])
                 plain_set[line.strip()][test][variant] = []
+                corrected_set[line.strip()][test][variant] = []
                 marker_set[line.strip()][test][variant] = []
                 papi_set[line.strip()][test][variant] = []
         testfp.close()
-fp.close()
+
 
 
 if len(test_set.keys()) == 0:
@@ -374,10 +419,12 @@ if not only_wiki:
             if test.startswith("REGEX"): continue
             file_plain = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_plain.dat")
             raw_plain = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_plain.raw")
+            file_correct = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_correct.dat")
             file_marker = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.dat")
             raw_marker = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.raw")
             outfp_plain = open(file_plain,'w')
             rawfp_plain = open(raw_plain,'w')
+            outfp_correct = open(file_correct,'w')
             outfp_marker = open(file_marker,'w')
             rawfp_marker = open(raw_marker,'w')
             if papi:
@@ -391,9 +438,11 @@ if not only_wiki:
             counter = 1
             for size in test_set[group][test]["variants"]:
                 if size.startswith("RUNS"): continue
-                bench_options = "-t %s -i %s -g 1 -w N:%s:1" % (test, test_set[group][test][size], size,)
+                bench_options = "-t %s -w S0:%s:1" % (test, size,)
                 for i in range(0,test_set[group][test]["RUNS"]):
                     # Run with plain likwid-bench
+                    print "*",
+                    sys.stdout.flush()
                     p = subprocess.Popen(bench_plain+" "+bench_options, shell=True, stdout=subprocess.PIPE,stderr=subprocess.STDOUT)
                     try:
                         p.wait()
@@ -402,11 +451,14 @@ if not only_wiki:
                     except:
                         sys.exit(1)
                     for line in stdout.split("\n"):
-                        if p.returncode != 0: print line
+                        #if p.returncode != 0: print line
                         match = test_set[group]["REGEX_BENCH"].match(line)
                         if match:
+                            value = float(match.group(1)) * test_set[group][test]["WA_FACTOR"]
                             plain_set[group][test][size].append(match.group(1))
+                            corrected_set[group][test][size].append(str(value))
                             outfp_plain.write(str(counter)+" "+match.group(1)+"\n")
+                            outfp_correct.write(str(counter)+" "+str(value)+"\n")
                         rawfp_plain.write(line+"\n")
                     # Run with papi instrumented likwid-bench
                     if papi:
@@ -419,7 +471,7 @@ if not only_wiki:
                         except:
                             sys.exit(1)
                         for line in stdout.split("\n"):
-                            if p.returncode != 0: print line
+                            #if p.returncode != 0: print line
                             match = test_set[group]["REGEX_PAPI"].match(line)
                             if match:
                                 papi_set[group][test][size].append(match.group(1))
@@ -435,99 +487,107 @@ if not only_wiki:
                     except:
                         sys.exit(1)
                     for line in stdout.split("\n"):
-                        if p.returncode != 0: print line
+                        #if p.returncode != 0: print line
                         match = test_set[group]["REGEX_PERF"].match(line)
                         if match:
                             marker_set[group][test][size].append(float(match.group(1)))
                             outfp_marker.write(str(counter)+" "+str(float(match.group(1)))+"\n")
                         rawfp_marker.write(line+"\n")
                     counter += 1
+                print("")
             outfp_plain.close()
             rawfp_plain.close()
+            outfp_correct.close()
             outfp_marker.close()
             rawfp_marker.close()
             if papi:
                 outfp_papi.close()
                 rawfp_papi.close()
-            if out_pgf: pgf_file = write_pgf(group, test, file_plain, file_marker, file_papi, script=script)
-            if out_gnuplot: plot_file = write_gnuplot(group, test, file_plain, file_marker, file_papi, script=script)
-            if out_grace: grace_file = write_grace(group, test, file_plain, file_marker, file_papi, script=script)
+            if out_pgf: pgf_file = write_pgf(group, test, file_plain, file_correct, file_marker, file_papi, script=script)
+            if out_gnuplot: plot_file = write_gnuplot(group, test, file_plain, file_correct, file_marker, file_papi, script=script)
+            if out_grace: grace_file = write_grace(group, test, file_plain, file_correct, file_marker, file_papi, script=script)
 
 
     script.close()
     os.chmod(scriptfile, stat.S_IRWXU)
-#if only_wiki:
-#    for group in test_set.keys():
-#        for test in test_set[group].keys():
-#            if test.startswith("REGEX"): continue
-#            filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_plain.dat")
-#            for i,size in enumerate(test_set[group][test]["variants"]):
-#                start = i*test_set[group][test]["RUNS"]
-#                end = (i+1)*test_set[group][test]["RUNS"]
-#                runs = test_set[group][test]["RUNS"]
-#                print "Read file %s for size %s from %d to %d" % (filename,size, start, end,)
-#                plain_set[group][test][size] = get_values_from_file(filename, start, runs)
-#                if len(plain_set[group][test][size]) == 0: plain_set[group][test][size].append(0)
-#            filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.dat")
-#            for i,size in enumerate(test_set[group][test]["variants"]):
-#                start = i*test_set[group][test]["RUNS"]
-#                end = (i+1)*test_set[group][test]["RUNS"]
-#                runs = test_set[group][test]["RUNS"]
-#                print "Read file %s for size %s from %d to %d" % (filename,size, start, end,)
-#                marker_set[group][test][size] = get_values_from_file(filename, start, runs)
-#                if len(marker_set[group][test][size]) == 0: marker_set[group][test][size].append(0)
+if only_wiki:
+    for group in test_set.keys():
+        for test in test_set[group].keys():
+            if test.startswith("REGEX"): continue
+            filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_plain.dat")
+            for i,size in enumerate(test_set[group][test]["variants"]):
+                start = i*test_set[group][test]["RUNS"]
+                end = (i+1)*test_set[group][test]["RUNS"]
+                runs = test_set[group][test]["RUNS"]
+                print "Read file %s for size %s from %d to %d" % (filename,size, start, end,)
+                plain_set[group][test][size] = get_values_from_file(filename, start, runs)
+                if len(plain_set[group][test][size]) == 0: plain_set[group][test][size].append(0)
+            filename = os.path.join(os.path.join(resultfolder,hostname),group+"_"+test+"_marker.dat")
+            for i,size in enumerate(test_set[group][test]["variants"]):
+                start = i*test_set[group][test]["RUNS"]
+                end = (i+1)*test_set[group][test]["RUNS"]
+                runs = test_set[group][test]["RUNS"]
+                print "Read file %s for size %s from %d to %d" % (filename,size, start, end,)
+                marker_set[group][test][size] = get_values_from_file(filename, start, runs)
+                if len(marker_set[group][test][size]) == 0: marker_set[group][test][size].append(0)
 
 
 if wiki or only_wiki:
     name, sockets, corespersocket, threadspercore = get_system_info();
     groups = get_groups()
     testable_groups = get_test_groups(groups)
-    #print groups
-    #print testable_groups
-    #if testable_groups.has_key("FLOPS_DP"): del testable_groups["FLOPS_DP"]
+    if testable_groups.has_key("FLOPS_DP"): del testable_groups["FLOPS_DP"]
 
-    print "#summary Accuracy Tests for %s\n" % (name,)
-    print "= Hardware description ="
+    print "# Accuracy Tests for %s\n" % (name,)
+    print "## Hardware description"
     print "Sockets: %d<br>" % (sockets,)
     print "Cores per socket: %d<br>" % (corespersocket,)
     print "Threads per core: %d<br>" % (threadspercore,)
     print "Total number of processing units: %d<br>" % (sockets * corespersocket * threadspercore)
     print
-    print "= Available groups ="
-    print "Each architecture defines a different set of groups. Here all the groups available for the %s are listed:<br>" % (name,)
+    print "## Available groups"
+    print "Each architecture defines a different set of performance groups. These groups help users to measure their derived metrics. Besides the event and counter defintion, a performance groups contains derived metrics that are calculated based on the measured data.<br>Here all the groups available for the %s are listed:<br>\n" % (name,)
+    print "| Name | Description |"
+    print "| ---- | ----------- |"
     for grp in groups.keys():
-        print "%s: %s<br>" % (grp, groups[grp],)
+        print "| %s | %s |" % (grp, groups[grp],)
     print
-    print "= Available verification tests ="
-    print "Not all groups can be tested for accuracy. Here only the groups are listed that can be verified. Each group is followed by the low-level benchmarks that are performed for comparison.<br>"
+    print "## Available verification tests"
+    print "Not all groups can be tested for accuracy. We don't have a test application for each performance group. Here only the groups are listed that can be verified. Each group is followed by the low-level benchmarks that are performed for comparison.<br>\n"
     #print testable_groups
+    print "| Group | Tests |"
+    print "|-------|-------|"
     for grp in testable_groups.keys():
-        print "%s: %s<br>" % (grp, ", ".join (testable_groups[grp]))
+        print "| %s | %s |" % (grp, ", ".join (testable_groups[grp]))
     print
-    print "= Accuracy comparison ="
-    print "For each varification group, the tests are performed twice. Once in a plain manner without measuring but calculating the resulting values and once through an instumented code with LIKWID.<br>"
+    print "## Accuracy comparison"
+    print "For each varification group, the tests are performed twice. Once in a plain manner without measuring but calculating the resulting values and once through an instumented code with LIKWID.<br>\n"
     
     
     for grp in testable_groups.keys():
-        print "== Verification of Group %s ==" % (grp,)
+        print "### Verification of Group %s" % (grp,)
         for test in testable_groups[grp]:
             #print grp, test, test_set[grp][test]
-            print "=== Verification of Group %s with Test %s ===" % (grp, test,)
-            print "|| *Stream size* || *Iterations* ||"
+            print "#### Verification of Group %s with Test %s\n" % (grp, test,)
+            print "| *Stream size* | *Iterations* |"
+            print "|---------------|--------------|"
             for variant in test_set[grp][test]["variants"]:
-                print "|| %s || %s ||" % (variant, test_set[grp][test][variant], )
+                print "| %s | %s |" % (variant, test_set[grp][test][variant], )
             print 
-            print "Each data size is tested %d times, hence the first %d entries on the x-axis correspond to the %d runs for the first data size of %s and so on.<br>" % (test_set[grp][test]["RUNS"],test_set[grp][test]["RUNS"],test_set[grp][test]["RUNS"],test_set[grp][test]["variants"][0],)
-            print "%s/accuracy/%s/%s_%s.png" % (picture_base,hostname, grp, test,)
+            print "Each data size is tested %d times, hence the first %d entries on the x-axis correspond to the %d runs for the first data size of %s and so on.<br>\n" % (test_set[grp][test]["RUNS"],test_set[grp][test]["RUNS"],test_set[grp][test]["RUNS"],test_set[grp][test]["variants"][0],)
+            print "%s/images/%s/%s_%s.png" % (picture_base,hostname, grp, test,)
             print
             file_plain = os.path.join(os.path.join(resultfolder,hostname),grp+"_"+test+"_plain.dat")
             file_marker = os.path.join(os.path.join(resultfolder,hostname),grp+"_"+test+"_marker.dat")
-            print "|| Variant || Plain (Min) || LIKWID (Min) || Plain (Max) || LIKWID (Max) || Plain (Avg) || LIKWID (Avg) ||"
+            print "| Variant | Plain (Min) | LIKWID (Min) | Plain (Max) | LIKWID (Max) | Plain (Avg) | LIKWID (Avg) |"
+            print "| ------- | ------- | ------- | ------- | ------- | ------- | ------- |"
             for i, variant in enumerate(test_set[grp][test]["variants"]):
                 results_plain = get_values_from_file(file_plain, i*test_set[grp][test]["RUNS"], test_set[grp][test]["RUNS"])
+                results_correct = get_values_from_file(file_correct, i*test_set[grp][test]["RUNS"], test_set[grp][test]["RUNS"])
                 results_marker = get_values_from_file(file_marker, i*test_set[grp][test]["RUNS"], test_set[grp][test]["RUNS"])
-                 if results_plain == []: results_plain.append(0)
-                 if results_marker == []: results_marker.append(0)
-                 print "|| %s || %d || %d || %d || %d || %d || %d ||" % (variant, min(results_plain), min(results_marker), max(results_plain), max(results_marker), int(statistics.mean(results_plain)), int(statistics.mean(results_marker)),)
+                if results_plain == []: results_plain.append(0)
+                if results_marker == []: results_marker.append(0)
+                if results_correct == []: results_correct.append(0)
+                print "| %s | %d | %d | %d | %d | %d | %d |" % (variant, min(results_correct), min(results_marker), max(results_correct), max(results_marker), int(statistics.mean(results_correct)), int(statistics.mean(results_marker)),)
             print
             print
